  PROGRAM moco_morgan
!    programme MOCO_MORGAN - 
!    contaminants metalliques et organiques dans un becher
!     Version 2 avec MOCO - septembre 2009 - converti en Fortran 90
!
!  Moco a compiler avec l option -02
! ________________________________________________________

USE parametres
USE com_morgan
#ifdef key_moco
  USE com_moco
  USE modul_moco,  ONLY : alloc_moco,MOCO
#endif
USE com_varmilieu
USE lectur_conta, ONLY : lect_1conta,lect_contavar
USE lectur_milieu, ONLY : lect_varmilieusens,lect_varmilieu
USE reactions, ONLY : sorption_cin,volatil,partage,sink_source
USE resultat, ONLY : init_ficres,ecri_result_scen,ecri_result_temps 

   
IMPLICIT NONE

integer                     :: i,iv,iesp,npas,iter,ine,ici
integer                     :: isu,ip,isub,it,ic,is,ilig,ificres
INTEGER                     :: nesp_disseq_lu,nesp_dissne_lu,nesp_parteq_lu
INTEGER                     :: nesp_partci_lu,nesp_partne_lu
LOGICAL                     :: l_domnl_lu
character(len=3)            :: modeln
character(len=1)            :: choix
real(kind=rlg)              :: dtchi,dtrest,dtsurvoleau,cc,fsom_nl,expo
real(kind=rlg)              :: sss,ss,ctot,difctot,cin_depo
real(kind=rlg)              :: somd,somp,tempj,xpartgl,perte_conta
real(kind=rlg),dimension(:),allocatable     :: somsink
real(kind=rlg),dimension(:,:),allocatable   :: c_anc
     
      
!  definition du contaminant et du type de simulation
!   par l'intermediaure d'une namelist
!   ================================
call init_nml

if(l_mode1c)then
     
!   variation des conditions du milieu pour un seul contaminant
!   on lit un fichier de parametres specifique du contaminant
!    et on lit des conditions du milieu qui peuvent varier
!   =========================================================
   write(*,*)'UN seul contaminant considere '
   write(*,*)'SENSIBILITE AUX CONDITIONS DU MILIEU '
   call lect_1conta
   call lect_varmilieusens
     
!   OU variation des parametres caracterisant le contaminant
!      pour tude de sensibilite sur ces parametres
!   ========================================================
else 
   write(*,*)'les conditions du milieu sont fixees '
   write(*,*)'SENSIBILITE AUX parametres du contaminant '
   call lect_contavar
   call lect_varmilieu
endif
            
!  ouverture et ecriture des premiers commentaires
!    dans le fichier resultat 
!   =================================
npas=echel_temps/dt
call init_ficres(npas)
      
!  initialisation des cumuls de transformations
!   =================================
allocate(smass_vol(ncalcul))
allocate(smass_flin(nvar,ncalcul))
allocate(smass_flout(nvar,ncalcul))
allocate(smass_part(nvar,ncalcul))
allocate(smass_transf(nvar,ncalcul))
allocate(smass_depo(nvar,ncalcul))
do i=1,ncalcul
  smass_vol(i)=0.
  do iv=1,nvar
        smass_flin(iv,i)=0.
        smass_flout(iv,i)=0.
        smass_part(iv,i)=0.
        smass_transf(iv,i)=0.
        smass_depo(iv,i)=0.
   enddo
enddo
dtsurvoleau=dt/vol_eau

#ifdef key_moco
!  initialisation des ligands et des phases pour MOCO
!   ====================================================
call alloc_moco
if(l_mode1c)then 
  allocate(actHplus(ncalcul))
  allocate(clig(nlig,ncalcul))
  allocate(cphas(nphasp,ncalcul))
else
  allocate(actHplus(1))
  allocate(clig(nlig,1))
  allocate(cphas(nphasp,1))
endif
if(l_moco_op)then
     cphas(1,:)=var_milieu(1,:)
else
   do ip=1,nphasp
     cphas(ip,:)=csub_v(isub_phase(ip),:)
   enddo
endif
 
#endif 
     
!   partage au temps initial 
!   calcul des concentrations de chaque espece particulaire 
!   subissant une sorption/desorption (phases changeables seules)
!   partage dissous/particulaire a l equilibre
!   ===================================================================
allocate(c_conta_tot(ncalcul))
allocate(c_conta_ech(ncalcul))
if(l_mode1c)then
  c_conta_tot(1:ncalcul)=var_milieu(8,1:ncalcul)
else
  c_conta_tot(1:ncalcul)=var_milieu(8,1)
endif
  
do i=1,ncalcul
  cc=sum(cesp(nesp_diss+nesp_part_eq+1:nespt,i))
  cesp(nvar,i)=c_conta_tot(i)-cc
  c_conta_ech(i)=cesp(nvar,i)
enddo
       
!   calcul des fractions de domaine lineaires et non lineaires
!   ===========================================================
#if ! defined key_moco
  allocate(f_dom_l(nespt))
  f_dom_l(1:nespt)=1.
  if(l_dom_nl)THEN
    do iesp=nesp_diss+1,nesp_diss+nesp_part_eq
      fsom_nl=sum(f_dom_nl(iesp,1:n_dom_nl(iesp)))
      f_dom_l(iesp)=1-fsom_nl
    enddo
  endif
#endif

!  allocation de tableaux
!   ========================
allocate(xpartg(ncalcul))
if(nb_reac>0)then
   allocate(somsink(nvar))
   allocate(sksc(nb_reac,nvar))
endif
       
!   PARTAGE initial
!   ==============
#ifdef key_moco
  call MOCO
#else
  call partage
#endif
       
!   calcul des masses echanges et reinitialisation des concentrations
!   ==================================================================
allocate(c_anc(nvar,ncalcul))
do i=1,ncalcul
  c_anc(1:nvar,i)=cesp(1:nvar,i)
  c_conta_tot(i)=sum(cesp(1:nvar,i))
enddo
       
!   calcul en fonction du temps
!   ===========================
t=0.
do it=1,npas
  t=t+dt
       
!   calcul du flux de transformation par 
!   hydrolyse, photolyse biodegradation
!   traitees par reactions generiques type MET&OR
!   ===============================================
!   quand on traitera avec des formules plus sophistiques
!   on appellera des subroutines       call hydrolyse ...
!  
  if(nb_reac>0) then 
    do i=1,ncalcul
      dtchi=dt
      dtrest=dt
      iter=1
      do while (iter<=200 .and. dtrest>1.)  
        call sink_source(i)
        do iv=1,nvar
          somsink(iv)=sum(sksc(1:nb_reac,iv))
!!!!!!!!!!!! traitement si valeurs negatives (trop de puits)
          if(somsink(iv)<0.)then
             dtchi=min(dtchi,-cesp(iv,i)/somsink(iv))
             if(dtchi==0._rlg)dtchi=dtrest
          endif
        enddo
        cesp(1:nvar,i)=cesp(1:nvar,i)+somsink(1:nvar)*dtchi
        dtrest=dtrest-dtchi
        dtchi=dtrest
        iter=iter+1
        if(iter==200)then
            write(*,*)'ATTENTION le pas de temps est trop grand'
            write(*,*)'pour pouvoir reproduire des reactions si rapides'
            write(*,*)'DIMINUER LE PAS DE TEMPS dans le fichier'
            if(l_mode1c)then
              write(*,*)name_file_varmilsens
            else
              write(*,*)name_file_varmilfix
            endif
            STOP
         endif
      enddo
      smass_transf(1:nvar,i)=smass_transf(1:nvar,i)+   &
                             cesp(1:nvar,i)-c_anc(1:nvar,i)
      c_anc(1:nvar,i)=cesp(1:nvar,i)
      c_conta_tot(i)=sum(cesp(1:nvar,i))
    enddo
  endif
        
!  calcul de la volatilisation sur l'espece dissoute libre
!  une seule espece volatilisable actuellement posible
!   =====================================================
!!!  var_milieu(6,i)=profondeur
!!!  var_milieu(7,i)=conta air
  if(l_volat)then
     do ic=1,ncalcul
        call volatil(ic)
        if(l_mode1c)then
          expo=exp(-dt*xkol/var_milieu(6,ic))
          cesp(iesp_volat,ic)=cesp(iesp_volat,ic)*expo+   &
                           var_milieu(7,ic)/henry_adimi*(1-expo)
        else
          expo=exp(-dt*xkol/var_milieu(6,1))
          cesp(iesp_volat,ic)=cesp(iesp_volat,ic)*expo+   &
                           var_milieu(7,1)/henry_adimi*(1-expo)
        endif
     enddo 
  
! calcul des masses echanges et reinitialisation des concentrations
! ===================================================================
     do i=1,ncalcul
       smass_vol(i)=smass_vol(i)+cesp(iesp_volat,i)-c_anc(iesp_volat,i)
       c_anc(iesp_volat,i)=cesp(iesp_volat,i)
       c_conta_tot(i)= sum(cesp(1:iesp_volat,i))
     enddo 
  endif 
            
!   prise en compte d'un flux entrant
!   ===============================================
  do i=1,ncalcul
    if(l_mode1c)then 
      cpart_v(1:npart,i)=cpart_v(1:npart,i)+flux_part_in(1:npart)*dtsurvoleau
      var_milieu(1,i)=sum(cpart_v(1:npart,i))
     if(var_milieu(1,i)==0.)then
      fpart_v(1:npart,i)=0.
     else
      fpart_v(1:npart,i)=cpart_v(1:npart,i)/var_milieu(1,i)
     endif
      csub_v(1:nsub,i)=csub_v(1:nsub,i)+flux_sub_in(1:nsub)*dtsurvoleau
    else
      cpart(1:npart)=cpart(1:npart)+flux_part_in(1:npart)*dtsurvoleau
      c_MESt= sum(cpart(1:npart))
      if(c_MESt==0.)then
        fpart(1:npart)=0.
      else
        fpart(1:npart)=cpart(1:npart)/c_MESt
      endif
      csub_v(1:nsub,1)=csub_v(1:nsub,1)+flux_sub_in(1:nsub)*dtsurvoleau
    endif
    c_conta_tot(i)=0.
    do ine=1,nesp_part_ne
      iesp=ine+nesp_diss+nesp_part_eq+nesp_part_ci
      cesp(iesp,i)=cesp(iesp,i)+flux_espne_in(ine)*dtsurvoleau
      smass_flin(iesp,i)=smass_flin(iesp,i)+flux_espne_in(ine)*dtsurvoleau
      c_conta_tot(i)=c_conta_tot(i)+cesp(iesp,i)
      c_anc(iesp,i)=cesp(iesp,i)
    enddo
    do ici=1,nesp_part_ci
      iesp=ici+nesp_diss+nesp_part_eq
      cesp(iesp,i)=cesp(iesp,i)+flux_espci_in(ici)*dtsurvoleau
      smass_flin(iesp,i)=smass_flin(iesp,i)+flux_espci_in(ici)*dtsurvoleau
      c_conta_tot(i)=c_conta_tot(i)+cesp(iesp,i)
      c_anc(iesp,i)=cesp(iesp,i)
    enddo
    c_conta_ech(i)=0.
    c_conta_ech(i)=sum(cesp(nesp_diss_ne+1:nesp_diss+nesp_part_eq,i))
#ifdef key_moco
    if(l_echgcat)c_conta_ech(i)=c_conta_ech(i)+cesp(iesp_echgcat,i)
#endif
    cesp(nvar,i)=cesp(nvar,i)+flux_espech_in*dtsurvoleau
    c_conta_ech(i)=c_conta_ech(i)+cesp(nvar,i)
    smass_flin(nvar,i)=smass_flin(nvar,i)+flux_espech_in*dtsurvoleau
    c_conta_tot(i)=c_conta_tot(i)+c_conta_ech(i)
    c_anc(nvar,i)=cesp(nvar,i)
  enddo

!  de nouveau calcul des concentrations de chaque espece
!  partage dissous/particulaire aprs transformations
!   ===================================================
#ifdef key_moco
  call MOCO
#else
  call partage
#endif

!  calcul de la sorption/desorption en cinetique
!   ===================================================
  if(nesp_part_ci>0)call sorption_cin
          
! calcul des masses echanges par sorption a l equilibre ou pas
!   et reinitialisation des concentrations
!   ===================================================
  do i=1,ncalcul
    sss=0.
    do iv=1,nvar
      ss=cesp(iv,i)-c_anc(iv,i)
      smass_part(iv,i)=smass_part(iv,i)+ss
      sss=sss+ss                                 
      c_anc(iv,i)=cesp(iv,i)
    enddo
    ctot=c_conta_tot(i)
    c_conta_tot(i)= sum(cesp(1:nvar,i))
    difctot=abs(ctot-c_conta_tot(i))
!    if(sss>epsnl.or.difctot>epsnl)then
!           write(*,*)'iteration ',i,' temps = ',t/86400
!           write(*,*)'non conservatif ',difctot
!           write(*,*)'diffspart ',sss
!           write(*,*)'conc.total avt,apres',ctot,c_conta_tot(i)
!    endif
  enddo 

!   depot rsiduel  des particules   
!   =======================================
!   sensibilite au milieu
  if(l_mode1c)then
    do i=1,ncalcul
      do isu=1,nsubp
        is=isu+nsubd
        cin_depo=vdepo(ipart_sub(is))/var_milieu(6,i)
        csub_v(is,i)= csub_v(is,i)*exp(-cin_depo*dt)
      enddo
      cpart_v(1:npart,i)=cpart_v(1:npart,i)*exp(-vdepo(1:npart)/var_milieu(6,i)*dt)
      var_milieu(1,i)=sum(cpart_v(1:npart,i))
      if(var_milieu(1,i)==0.)then
        fpart_v(1:npart,i)=0. 
      else
        fpart_v(1:npart,i)=cpart_v(1:npart,i)/var_milieu(1,i)
      endif
!      do iesp=nesp_diss+1,nespt-ncat-nthiolp-nesp_part_ne
      do iesp=nesp_diss+1,nespt-ncat-nesp_part_ne
        isub=isub_esp(iesp)
        cin_depo=vdepo(ipart_sub(isub))/var_milieu(6,i) 
        cesp(iesp,i)=cesp(iesp,i)*exp(-cin_depo*dt)
        smass_depo(iesp,i)=smass_depo(iesp,i)+cesp(iesp,i)-c_anc(iesp,i)
        c_anc(iesp,i)=cesp(iesp,i)
      enddo
      do ine=1,nesp_part_ne
        iesp=nesp_diss+nesp_part_eq+nesp_part_ci+ine
        cin_depo=vdepo(ityp_part(iesp))/var_milieu(6,i) 
        cesp(iesp,i)=cesp(iesp,i)*exp(-cin_depo*dt)
        smass_depo(iesp,i)=smass_depo(iesp,i)+cesp(iesp,i)-c_anc(iesp,i)
        c_anc(iesp,i)=cesp(iesp,i)
      enddo

#ifdef key_moco
      if(l_echgcat)then
        iesp=iesp_echgcat
        cin_depo=vdepo(ipart_sub(isub_echcompcat))/var_milieu(6,i) 
        cesp(iesp,i)=cesp(iesp,i)*exp(-cin_depo*dt)
        smass_depo(iesp,i)=smass_depo(iesp,i)+cesp(iesp,i)-c_anc(iesp,i)
        c_anc(iesp,i)=cesp(iesp,i)
      endif
!      if(l_thiolpart)then
!        iesp=iesp_thiol_mono
!        cin_depo=vdepo(ipart_sub(isub_thiolpart))/var_milieu(6,i) 
!        cesp(iesp,i)=cesp(iesp,i)*exp(-cin_depo*dt)
!        smass_depo(iesp,i)=smass_depo(iesp,i)+cesp(iesp,i)-c_anc(iesp,i)
!        c_anc(iesp,i)=cesp(iesp,i)
!        iesp=iesp_thiol_bi
!        cesp(iesp,i)=cesp(iesp,i)*exp(-cin_depo*dt)
!        smass_depo(iesp,i)=smass_depo(iesp,i)+cesp(iesp,i)-c_anc(iesp,i)
!        c_anc(iesp,i)=cesp(iesp,i)
!      endif
#endif
      c_anc(1:nvar,i)=cesp(1:nvar,i)
      c_conta_tot(i)=sum(cesp(1:nvar,i))
    enddo

!  sensibilite au contaminant
  else
    cpart(1:npart)=cpart(1:npart)*exp(-vdepo(1:npart)/zc*dt)
    c_MESt= sum(cpart(1:npart))
    if(c_MESt==0.)then
      fpart(1:npart)=0.
    else
      fpart(1:npart)=cpart(1:npart)/c_MESt
    endif
    do isu=1,nsubp
        is=isu+nsubd
        csub_v(is,1)= csub_v(is,1)*exp(-vdepo(ipart_sub(is))/zc*dt)
    enddo
    do i=1,ncalcul
!      do iesp=nesp_diss+1,nespt-ncat-nthiolp-nesp_part_ne
      do iesp=nesp_diss+1,nespt-ncat-nesp_part_ne
        isub=isub_esp(iesp)
        cesp(iesp,i)=cesp(iesp,i)*exp(-vdepo(ipart_sub(isub))/zc*dt)
        smass_depo(iesp,i)=smass_depo(iesp,i)+cesp(iesp,i)-c_anc(iesp,i)
        c_anc(iesp,i)=cesp(iesp,i)
      enddo
      do ine=1,nesp_part_ne
        iesp=nesp_diss+nesp_part_eq+nesp_part_ci+ine
        cesp(iesp,i)=cesp(iesp,i)*exp(-vdepo(ityp_part(iesp))/zc*dt)
        smass_depo(iesp,i)=smass_depo(iesp,i)+cesp(iesp,i)-c_anc(iesp,i)
        c_anc(iesp,i)=cesp(iesp,i)
      enddo
#ifdef key_moco
      if(l_echgcat)then
        iesp=iesp_echgcat
        cin_depo=vdepo(ipart_sub(isub_echcompcat))/zc 
        cesp(iesp,i)=cesp(iesp,i)*exp(-cin_depo*dt)
        smass_depo(iesp,i)=smass_depo(iesp,i)+cesp(iesp,i)-c_anc(iesp,i)
        c_anc(iesp,i)=cesp(iesp,i)
      endif
!      if(l_thiolpart)then
!        iesp=iesp_thiol_mono
!        cin_depo=vdepo(ipart_sub(isub_thiolpart))/zc 
!        cesp(iesp,i)=cesp(iesp,i)*exp(-cin_depo*dt)
!        smass_depo(iesp,i)=smass_depo(iesp,i)+cesp(iesp,i)-c_anc(iesp,i)
!        c_anc(iesp,i)=cesp(iesp,i)
!        iesp=iesp_thiol_bi
!        cesp(iesp,i)=cesp(iesp,i)*exp(-cin_depo*dt)
!        smass_depo(iesp,i)=smass_depo(iesp,i)+cesp(iesp,i)-c_anc(iesp,i)
!        c_anc(iesp,i)=cesp(iesp,i)
!      endif
#endif
      c_conta_tot(i)=0.
      c_anc(1:nvar,i)=cesp(1:nvar,i)
      c_conta_tot(i)=sum(cesp(1:nvar,i))
    enddo
  endif

!  calcul des flux sortants 
!   ===================================================
  do i=1,ncalcul
    if(l_mode1c)then 
       cpart_v(1:npart,i)=cpart_v(1:npart,i)- flux_eau_inout*cpart_v(1:npart,i)*dtsurvoleau
       var_milieu(1,i)=sum(cpart_v(1:npart,i))
       if(var_milieu(1,i)==0.)then
        fpart_v(1:npart,i)=0.
       else
        fpart_v(1:npart,i)=cpart_v(1:npart,i)/var_milieu(1,i)
       endif
       csub_v(1:nsub,i)=csub_v(1:nsub,i)-flux_eau_inout*csub_v(1:nsub,i)*dtsurvoleau
    else
       cpart(1:npart)=cpart(1:npart)-flux_eau_inout*cpart(1:npart)*dtsurvoleau
       c_MESt= sum(cpart(1:npart))
       if(c_MESt==0.)then
        fpart(1:npart)=0.
       else
        fpart(1:npart)=cpart(1:npart)/c_MESt
       endif
       csub_v(1:nsub,1)=csub_v(1:nsub,1)-flux_eau_inout*csub_v(1:nsub,1)*dtsurvoleau
    endif
    cesp(1:nvar,i)=cesp(1:nvar,i)-flux_eau_inout*cesp(1:nvar,i)*dtsurvoleau
    smass_flout(1:nvar,i)=smass_flout(1:nvar,i)+cesp(1:nvar,i)-c_anc(1:nvar,i)
    c_anc(1:nvar,i)=cesp(1:nvar,i)
    c_conta_tot(i)=sum(cesp(1:nvar,i))
  enddo
        
#ifdef key_moco
!  reactualisation des ligands et des phases pour MOCO
!   ====================================================
if(l_moco_op)then
   cphas(1,:)=var_milieu(1,:)
else
   do ip=1,nphasp
     cphas(ip,:)=csub_v(isub_phase(ip),:)
   enddo
endif
#endif 
        
! ecriture des resultats tous les pas de temps ecriture pour chaque scenario
!   =======================================================================
  if(dmod(t,dt_ecr)<dt)then
         
!!!!!  calcul du coefficient de partage vraiment global 
!!!!!   (y compris especes particulaires non echangeables)
    do i=1,ncalcul
      if(nesp_diss==1)then
          somd=cesp(nvar,i)+cesp(1,i)
      else
          somd=cesp(nvar,i)+sum(cesp(1:nesp_diss,i))
      endif
      somp=sum(cesp(nesp_diss+1:nespt,i))
      if(l_mode1c)then
        if(var_milieu(1,i)==0.)then
         xpartg(i)=0.
        else
         xpartg(i)=somp/somd/var_milieu(1,i)
        endif
      else 
        if(c_MESt==0.)then
         xpartg(i)=0.
        else
         xpartg(i)=somp/somd/c_MESt
        endif
      endif
    enddo
           
    tempj=t/86400
     if(xpartg(1)>0)then
         xpartgl=log10(xpartg(1)*1000)
     else if(xpartg(1)<0.)then
        write(*,*)'Coef de Partage negatif ===>> PROBLEME'
        write(*,*)'egal a :',xpartg(1)
        xpartgl=0.
     else
        xpartgl=0.
     endif       
     call ecri_result_scen     
   endif

! ecriture des resultats de tous les scenarios  des instants choisis
!   =======================================================================
  if(ncalcul > 1 .and. (t==dt .or. dmod(t,dt_ecr_scen)<dt))then
     if(t==dt)ificres=260
         
!!!!!  calcul du coefficient de partage vraiment global 
!!!!!   (y compris especes particulaires non echangeables)
    do i=1,ncalcul
      if(nesp_diss==1)then
          somd=cesp(nvar,i)+cesp(1,i)
      else
          somd=cesp(nvar,i)+sum(cesp(1:nesp_diss,i))
      endif
      somp=sum(cesp(nesp_diss+1:nespt,i))
      if(l_mode1c)then
        if(var_milieu(1,i)==0.)then
          xpartg(i)=0.
        else
         xpartg(i)=somp/somd/var_milieu(1,i)
        endif
      else 
        if(c_MESt==0.)then
         xpartg(i)=0.
        else
         xpartg(i)=somp/somd/c_MESt
        endif
      endif
    enddo
           
    tempj=t/86400
    write(*,*)'RESULTATS au temps t=',t,'s',tempj,' jours'
!    write(*,*)'concentration et % de lespece dissoute libre=',  &
!                   cesp(nvar,1),pcent_cesp(nvar,1),'%'
!    do iesp=1,nespt
!      write(*,*)'concentration et % de ',name_esp(iesp),        &
!                 cesp(iesp,1),pcent_cesp(iesp,1),'%'
!      if(iesp.gt.nesp_diss)then
!         cespp=cesp(iesp,1)/c_MESt
!         write(*,*)'concentration en masse/kg de',name_esp(iesp),cespp
!      endif
!    enddo
!    write(*,*)'Coef de partage global =',xpartg(1)
!    xpartgl=log10(xpartg(1))
!    write(*,*)'Coef de partage global (log10 l/g) =',xpartgl
     if(xpartg(1)>0)then
         xpartgl=log10(xpartg(1)*1000)
     else
        write(*,*)'Coef de Partage negatif ===>> PROBLEME'
        xpartgl=0.
     endif
     write(*,"(a,F8.2)")'Coef de partage global (log10 ml/g) =',xpartgl
     write(*,"(a,g10.2)")'concentration totale finale',c_conta_tot(1)
     perte_conta=(var_milieu(8,1)-c_conta_tot(1))/var_milieu(8,1)*100
     write(*,"(a,F8.2,a)")'perte de ',perte_conta,' % par rapport  t0'        
       
     ificres=ificres+1
     call ecri_result_temps(ificres)     
   endif
!  fin de la boucle sur le temps
enddo

END PROGRAM moco_morgan
     
