 MODULE modul_moco
#ifdef key_moco

   !!======================================================================
   !!                   ***  MODULE  modul_MOCO           ***
   !!    contient les subroutines decrivant la complexation de surface
   !!      et l echange cationique (module MOCO)
   !!   aunsi que la resolution des equations par Levenberg Marquardt
   !!======================================================================
   !! * Modules used
USE parametres
USE com_moco
USE com_morgan
USE com_varmilieu

IMPLICIT NONE
PRIVATE

   !! * Accessibility
PUBLIC alloc_moco     ! routine called by moco_morgan.F90
PUBLIC MOCO     ! routine called by moco_morgan.F90

   !! * Shared module variables
integer                                     :: nsubc,nsubsitc,ncomp,nequtc
integer,dimension(:),allocatable            :: nequsc,nsubsic,nsitc,jsubs
integer,dimension(:),allocatable            :: nequs,nsubs
real(kind=rlg),dimension(:),allocatable     :: xcsub,xact,xesp,csubs
real(kind=rlg),dimension(:),allocatable     :: xrfsub,xchasu
real(kind=rlg)                              :: xsokcat,dixph,xg2shp,xgcat
real(kind=rlg)                              :: x23i,xsom,xctot
real(kind=rlg)                              :: crsh,xrshg,xrshg2,crsmoins
real(kind=rlg),dimension(:),allocatable       :: xchasuc
real(kind=rlg),dimension(:,:),allocatable     :: xsitac,csitac,socat
real(kind=rlg),dimension(:,:),allocatable     :: xka1,xka2

   !! * Private variables

 CONTAINS
                SUBROUTINE alloc_moco
!&E---------------------------------------------------------------------
!         allocation de tableaux propres  moco    
!&E---------------------------------------------------------------------
   !! * Modules used

   !! Arguments

   !! * Local declarations

   if(nlig>0)allocate(xactl(nlig))
   if(nphasp>0)then
     allocate(jsubs(nphasp))
     allocate(csubs(nphasp))
     allocate(nequsc(nphasp))
     allocate(nsitc(nphasp))
     allocate(nsubsic(nphasp))
     allocate(xsitac(nsitmoco_max,nphasp))
     allocate(csitac(nsitmoco_max,nphasp))
     allocate(xcsub(nphasp))
     allocate(xrfsub(nphasp))
     allocate(xka1(nsitmoco_max,nphasp))
     allocate(xka2(nsitmoco_max,nphasp))
     allocate(xk2rsh(nphasp))
     allocate(xchasu(nphasp))
     allocate(xchasuc(nphasp))
    else  
     allocate(jsubs(1))
     allocate(csubs(1))
     allocate(nequsc(1))
     allocate(nsitc(1))
     allocate(nsubsic(1))
     allocate(xsitac(1,1))
     allocate(csitac(1,1))
     allocate(xcsub(1))
     allocate(xrfsub(1))
     allocate(xka1(1,1))
     allocate(xka2(1,1))
     allocate(xk2rsh(1))
     allocate(xchasu(1))
     allocate(xchasuc(1))
    endif
    allocate(socat(nvar,ncalcul))
    if(nesp_diss_moco>0)then
      allocate(xact(nesp_diss_moco))
      allocate(xesp(nesp_diss_moco))
    endif
! initalisation
   csitac(:,:)=0.d00
   xsitac(:,:)=0.d00
   xka1(:,:)=0.d00
   xka2(:,:)=0.d00
   
END SUBROUTINE alloc_moco

!=========================================================================
                SUBROUTINE cal_cligdiss(i)
!&E---------------------------------------------------------------------
!         calcul des concentrations de ligands dissous     
!&E---------------------------------------------------------------------
   !! * Modules used

   !! Arguments
integer,INTENT(IN)            :: i

   !! * Local declarations
integer                :: ilig
   
  actHplus(i)=10**(-var_milieu(9,i))
  do ilig=1,nlig

    if(ilig==ilisal)then
!!!!          calcul de la chlorinite  en fonction de la salinite
!!            chlorinite en mole/l ; salinite en g/l (0.4856/35=0.0138743)
       clig(ilisal,i)=0.0138743_rlg*var_milieu(3,i)
     
    else if(ilig==iliph)then
!!!!          ligand OH-  a partir du pH
!!!!          activite H+ ss dim= est egale a g(1)*[H]  pH=varmilieu(9)
       clig(iliph,i)=1.d-14/actHplus(i)/xactl(iliph)
     
    else if(ilig==ilithiold)then
!             ligand organique (en concentration en mole/l )
!             calcul du radical RS- 
!             Thiols=RSH+RS-   et Ka=g1. RS- . (H+) /RSH
!                           donc  RSH=  g1. RS- . (H+) /Ka
!                           donc Thiols=g1. RS- . (H+) /Ka + RS-
!                           donc RS- = Thiols / (1+g1 (H+) / Ka)
       clig(ilithiold,i)=csub_v(isub_lig(ilig),i)/    &
                (1+actHplus(i)/xkathiol*xactl(ilithiold))
                
    else if(ilig==ilithiolp)then
!             ligand organique particulaire considere ici comme du dissous
!           en premiere approximation car les donnes pour les traiter 
!          comme des sites ou phase particulaires ne sont pas accessibles
!             (concentration totale donnee en mole/g )
!             calcul du radical RSp-  - mme constante Kah que pour le dissous
!             Thiolsp=RSHp+RSp-   et Ka=g1. RSp- . (H+) /RSHp
!                           donc  RSHp=  g1. RSp- . (H+) /Ka
!                           donc Thiolsp=g1. RSp- . (H+) /Ka + RSp-
!                           donc RSp- = Thiolsp / (1+g1 (H+) / Ka)
       clig(ilithiolp,i)=csub_v(isub_lig(ilig),i)*var_milieu(1,i)/    &
                (1+actHplus(i)/xkathiol*xactl(ilithiolp))

    else if(ilig==ilico3)then
       clig(ilico3,i)=10**xkaCO3*csub_v(isub_lig(ilico3),i)   &
                              /actHplus(i)*xactl(ilico3)
                          
    else if(ilig==ilisulf)then
!             ligand sulfure HS- calcul  partir des sulfures totaux
!             calcul de HS- = Stot/(1+((H+)g1/K1S)+(K2Sg1/(H+)g2))
!   3  sulfur : HS- H2S et S2- on prend ici la charge du sulfur S2-
!       g(1)=xactl(ilisulf1[HS-])=xactl1
!       g(2)=xactl(ilisulf2[S2-])=xactl2 
       clig(ilisulf,i)=csub_v(isub_lig(ilig),i)/    &
                (1+actHplus(i)*xactl1/xka1S +xka2S/actHplus(i)*xactl1/xactl2)

    else if(ilig==ilimodcarb)then
!             ligand MOD carboxylique calcul  partir des HLcarb totaux
!             calcul de HL = HLtot/(1+(KaL/(H+)g1))
       clig(ilimodcarb,i)=csub_v(isub_lig(ilig),i)/    &
                         (1+xkaLcarb/actHplus(i)/xactl1)

    else if(ilig==ilimodphen)then
!             ligand MOD phenolique calcul  partir des HLcarb totaux
!             calcul de HL = HLtot/(1+(KaL/(H+)g1))
       clig(ilimodphen,i)=csub_v(isub_lig(ilig),i)/    &
                         (1+xkaLphen/actHplus(i)/xactl1)

    else
       clig(ilig,i)=csub_v(isub_lig(ilig),i)
    endif
                
  enddo

END SUBROUTINE cal_cligdiss

!=========================================================================
                SUBROUTINE MOCO
!&E---------------------------------------------------------------------
!         Partage dissous/particulaire pour les metaux
!       Complexation de surface - et echange cationique       
!&E---------------------------------------------------------------------
   !! * Modules used

   !! Arguments

   !! * Local declarations
integer                :: i,jsubc,js,jsc,iesp,ilig,ipotx,ivs,iesp_max
integer                :: ipot,ik,nikm,isit,isoh,in,ip,isohx,lwa,info
real(kind=rlg)         :: fionic,racic,factic,xacti,xactmet,xactcat,somet
real(kind=rlg)         :: sect,somoins,cdiss,difc,somethiol
real(kind=rlg)         :: cptot,catpart,a1,a2,cesp_max
real(kind=rlg),dimension(ndequ)            :: fvec,x,fver,xikm
real(kind=rlg),dimension(ndw)              :: wa
real(kind=rlg),dimension(ndequ,ndequ)      :: fjac
integer,dimension(ndequ)                   :: ipvt
real(kind=rlg),dimension(3)                :: xcoef


   do i=1,ncalcul
     if(l_mode1c)then
       ivs=i
     else
       ivs=1
     endif  
      
!      calcul des variables derivees 
!::::::::::::::::::::::::::::::::::::
!       activite H+ ss dim (recalcule pour au cas ou il n'y a pas de ligand OH)
!       est egale a g(1)*[H]  pH=varmilieu(9)
!        force ionique : fionic (formule de Fiol et al)  salinite=varmilieu(3)
     fionic=0.0029_rlg+(1.8575d-02*var_milieu(3,ivs))+ (1.639d-05*var_milieu(3,ivs)**2)
!       calcul des coefficients d activite pour les differentes especes et ligands
!       varient en fonction de la force ionique de chaque espece et de la charge
!::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
     racic=sqrt(fionic)
     factic=racic/(1+racic)

!       calcul du coefficient d'activite de l'espece libre metal i+
!::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
     xacti=-0.5*(factic-0.3*fionic)
     xactmet=10**(xacti*icharlib**2)

!       calcul du coefficient d'activite de chaque ligand
!::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
     xactl(1:nlig)=10**(xacti*icharl(1:nlig)**2)

!      ou calcul de xatl1 et xatcl2 pour charge1 et 2 (sert pur sulfur)
!::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
     xactl1=10**xacti
     xactl2=10**(xacti*4)
      
!      concentrations des ligands
!::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
     if(l_mode1c)then
       call cal_cligdiss(i)
     else
       ! pas de variation des ligands si etude de sensibilite sur parametres
       !  on ne les calcule qu une seule fois
       if(i==1)call cal_cligdiss(ivs)
     endif
     
!       coefficient d'activite du cation competiteur
!:::::::::::::::::::::::::::::::::::::::::::::::::::
     ncomp=0
     if(l_competcat .or. l_echgcat)then
        xactcat=10**(xacti*icharcat**2)
        if(l_competcat)ncomp=1
     endif
!
!   calcul de la constante pour les corrections de surface 
! Temperature (C)= var_milieu(2,ivs)
!:::::::::::::::::::::::::::::::::::::::::::::::::::
     xcorsurf=xfarad/cst_gas_univ/(var_milieu(2,ivs)+273.15)

!       calcul des coefficients utlises dans le systeme d'equations
!       non dependant du substrat
!:::::::::::::::::::::::::::::::::::::::::::::::::::
!        xg2shp exprime en L/nmole (xactmet ss dim  et actHplus(i) en mole/L  )
     xg2shp=xactmet*1.d-09/actHplus(ivs)
!        xgcat sans dimension (1/actHplus(i) exprime en l/mole, ccat en mole/l)
     xgcat=ncomp*xactcat*ccat/actHplus(ivs)
!         x23i : coefficient = 2.3 racine (I)
     x23i=2.3*racic
! activite Hplus dont on se sert aussi dans fcn mais attention en mole/L
     dixph=actHplus(ivs)

!       calcul des concentrations de substrats et des sites actifs sur substrat
!       calcul du nombre d'equations a resoudre par substrat et du nombre
!       total d'equations a resoudre si un csubs est nul
!         csubs en g/l - csitac en mole/l
!:::::::::::::::::::::::::::::::::::::::::::::::::::

     jsubc=0
     nequtc=1
     nsubc=nphasp
     do js=1,nphasp
       jsubc=jsubc+1
       if(cphas(js,ivs).lt.epsnl)then
         jsubc=jsubc-1
         nsubc=nsubc-1
       else
         jsubs(jsubc)=js
         csubs(jsubc)=cphas(js,ivs)
         nequsc(jsubc)=nsit(js)+1
         nsitc(jsubc)=nsit(js)
         nequtc=nequtc+nequsc(jsubc)
         if(.not.l_mode1c)then
           ssp(js)=sspv(js,i)
           do isit=1,nsit(js)
             sitac(isit,js)=sitacv(isit,js,i)
             xequa1(isit,js)=xequa1v(isit,js,i)
             xequa2(isit,js)=xequa2v(isit,js,i)
             xequms(isit,js)=xequmsv(isit,js,i)
           enddo
         endif
   ! sitac est en mole/m2 et csitac en mole/L
         do isit=1,nsit(js)
             csitac(isit,jsubc)=csubs(jsubc)*sitac(isit,js)*ssp(js)
         enddo
       endif
!           conversion de csubs pour prendre en compte la modif d'unite de x(isoh)
!              xcsub(js)=csubs(js)*1.70073d+05
!       xcsub(js)=csubs(js)*convnmol
       xcsub(js)=csubs(js)
     enddo

!       calcul du nombre indice pour chaque substrat en fonction du 
!       nombre de substrat et du nombre de sites pas substrat
     
     nsubsic(1)=0
     do  jsc=2,nsubc
        js=jsubs(jsc)
        nsubsic(jsc)=nsubsic(jsc-1)+nsit(js-1)
     enddo
     if(nsubc.ne.0)nsubsitc=nsubsic(nsubc)+nsit(nsubc)

!       boucle sur les especes complexees avec chaque ligand dissous
!       calcul de la somme en facteur de met2+ 
      xsom=1. 
      do iesp=1,nesp_diss_moco
        ilig=ilig_esp(iesp)
!          coefficient d'activite de l'espece
        xact(iesp)=10**(xacti*ichare(iesp)**2)
!          activite de chaque espece
!          xequ :ss dim; xactl en l/mole et clig en mole/l ==> xesp ss dim
        xesp(iesp)=xequ(iesp)*(xactl(ilig)*clig(ilig,ivs))**xfactl(iesp)   &
                   *actHplus(ivs)**(-xfacth_som(iesp))
!          somme des K*act*cl-
        xsom=xsom+xesp(iesp)*xactmet/xact(iesp)
      enddo
 
!      echange ou competition de cation :
      if(l_competcat .or.l_echgcat)then
!            nombre de sites total /l (echange de cation) en moles/l
!            sur le premier substrat particulaire seulement
         sect=cec*csub_v(isub_echcompcat,ivs)
!            coefficient d'activite du cation competiteur
         if(l_echgcat)then
!              SO-  (mole/l) (xsokcat sans dimension)
             somoins=sect/(1+xequcat*ccat*xactcat)
             xsokcat=somoins*xequmet*xactmet
         else
             somoins=0
             xsokcat=0.
         endif
      else
         somoins=0
         xsokcat=0.
      endif
!
!     complexation sur site thiol particulaire consideres comme des thiols dissous
!      if(l_thiolpart)then
!        crsh=cthiolpart*csub_v(isub_thiolpart,ivs)/(1+xkathiolp/actHplus(ivs))
!        crsmoins=xkathiolp*crsh/actHplus(ivs)
!        xrshg=xequRSHg*crsh*xg2shp
!        xrshg2=xequR2SHg*xg2shp/actHplus(ivs)*1.e-09*crsh**2
!      endif


!       si MES NUL ou inferieur au seuil : calcul des especes dissoutes seules
!       ou bien si echange cationique seul
      if(var_milieu(1,ivs).le.seuil_mes.or.nphasp.eq.0)then

!         calcul en concentration du met2+ (nmole/l)
        cesp(nvar,i)=c_conta_ech(i)/(xsom+xsokcat)
        cdiss=cesp(nvar,i)*xsom
        do ip=1,nphasp
          cesp(iespp_moco(ip),i)=0.
        enddo
    
! calcul des differentes especes dissoutes (en concentration)
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
        cesp(nesp_diss_ne+1:nesp_diss_ne+nesp_diss_moco,i)=  &
                               cesp(nvar,i)*xactmet*   &
                               xesp(1:nesp_diss_moco)/xact(1:nesp_diss_moco)

! calcul de l'espece particulaire dans le cas ou il y a echange cationique seul
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!   s'il n'y a pas de phases particulaire associee 
! a des especes paticulaires complexees
!   mais il peut y avoir une espece sur un substrat qui change des cations
        if(nphasp==0)then
          xctot=c_conta_ech(i)-cdiss
          somet=xctot
          if(l_echgcat)cesp(iesp_echgcat,i)=somet
          cptot=somet
        endif
       iesp_max=nvar

      else 
!      avec des MES suffisantes

!        calcul des coefficients utlises dans le systeme d'equations
!        dependant du substrat
        do jsc=1,nsubc
          js=jsubs(jsc)
!           xrfsub= Faraday en nmole/L/surf. specif. /1000  
          xrfsub(jsc)=xfaradn/ssp(js)/1000.
!           xka1, xka2 et xchasu ss dim
          do isit=1,nsit(js)
            if(ika1(isit,js)==1)then
              xka1(isit,jsc)=xequa1(isit,js)
            else
              xka1(isit,jsc)=1.
              idchars(isit,js,1)=1
            endif
            xka2(isit,jsc)=xequa2(isit,js)
          enddo
          xchasu(jsc)=icharlib-1
          xchasuc(jsc)=icharcat-1
!         xsitac : conc. totale en sites par particules en nmole/l 
          xsitac(1:nsit(js),jsc)=csitac(1:nsit(js),jsc)*1.d+09
        enddo

!  valeurs initiales estimees pour l'iteration
!   donnees par Jean Louis (2 particules 1 site)
!   comcentrations donnees en mole/l et potentiel en volts
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
        xctot=c_conta_ech(i)
        xinit(1)=xctot/(xsom+xsokcat)
        x(1)=xinit(1)
        xcoef(:)=1.e+05
        if(var_milieu(3,ivs)==0._rlg)x(1)=xinit(1)*1.d-05
!          Plomb
        if(TRIM(ADJUSTL(ADJUSTR(name_conta)))=='Pb' .or.  &
             TRIM(ADJUSTL(ADJUSTR(name_conta)))=='Plomb')then
           x(1)=xinit(1)*1.d+5
           if(var_milieu(9,ivs)>7.)x(1)=xinit(1)*1.d-10
        else if(TRIM(ADJUSTL(ADJUSTR(name_conta)))=='Hg' .or.  &
             TRIM(ADJUSTL(ADJUSTR(name_conta)))=='Mercure') then
               x(1)=xinit(1)*1.d+10
               xcoef(1:nsitmoco_max)=1.e-20 
        endif
        
        do jsc=1,nsubc
          js=jsubs(jsc)
          ipotx=1+nsubsit+js
          ipot=1+nsubsitc+jsc
          x(ipot)=xinit(ipotx)
          do isit=1,nsit(js)
            isohx=1+nsubsi(js)+isit
            isoh=1+nsubsic(jsc)+isit
            x(isoh)=xinit(isohx)
            a1=ika1(isit,js)*dixph/xka1(isit,jsc)
            a2=xka2(isit,jsc)/dixph
            x(isoh)=xsitac(isit,jsc)/csubs(js)/(1+a1+a2)*xcoef(isit)
            xinit(isohx)=x(isoh)
          enddo
        enddo

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  resolution des nequt equations non lineaires par levenberg
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	lwa=6*nequtc
 	call lmder1(nequtc,nequtc,x(1:nequtc),fvec(1:nequtc),  &
 	            fjac(1:nequtc,1:nequtc),nequtc,info,    &
 	            ipvt(1:nequtc),wa(1:lwa),lwa)

!  calcul des xikm
	ik=0
	do jsc=1,nsubc
	  js=jsubs(jsc)
	  ipot=1+nsubsitc+jsc
	  do isit=1,nsitc(js)
            ik=ik+1
            xikm(ik)=xequms(isit,js)*dexp(-xcorsurf*idcharm(isit,js)*x(ipot))
	  enddo
	enddo
	nikm=ik

!.........................................................................
!  verification des equations
!.........................................................................
       if(l_verifequ)call verif(nequtc,x(1:nequtc),fver(1:nequtc))

!  calcul en concentration du met2+(nmole/l)
        cesp(nvar,i)=x(1)
! calcul du metal dissous total (en nmole/l)
        cdiss=cesp(nvar,i)*xsom
! reperage de la concentration max 
        cesp_max= cesp(nvar,i)
        iesp_max=nvar
!
! calcul des differentes especes dissoutes (en concentration)
!----------------------------------------------------------------
        if(l_group_espdmoc)then
          cesp(iespd_moco(1),i)=0.
          do iesp=1,nesp_diss_moco
             cesp(iespd_moco(1),i)=  &
                       cesp(iespd_moco(1),i)+   &
                            cesp(nvar,i)*xactmet*xesp(iesp)/xact(iesp)
             if(cesp(iespd_moco(1),i) > cesp_max)then
               cesp_max= cesp(iespd_moco(1),i)
               iesp_max=iespd_moco(1)
             endif
          enddo
        else
          do iesp=1,nesp_diss_moco
            cesp(iespd_moco(iesp),i)= cesp(nvar,i)*xactmet*xesp(iesp)/xact(iesp)
            if(cesp(iespd_moco(iesp),i) > cesp_max)then
               cesp_max= cesp(iespd_moco(iesp),i)
               iesp_max=iespd_moco(iesp)
             endif
          enddo
        endif
        
! calcul du metal adsorbe sur les particules par echange cationique
!----------------------------------------------------------------
         somet=0.
         if(l_echgcat)then
           somet=xsokcat*x(1)
           cesp(iesp_echgcat,i)=somet
           if(cesp(iesp_echgcat,i) > cesp_max)then
               cesp_max= cesp(iesp_echgcat,i)
               iesp_max=iesp_echgcat
           endif
         endif

! calcul du metal complexe aux sites monodentate et bidentate des thiols particulaires
!----------------------------------------------------------------
!        somethiol=0.
!        if(l_thiolpart)then
!           cesp(iesp_thiol_mono,i)=xrshg*x(1)*dexp(-xcorsurf*x(ipot))
!           cesp(iesp_thiol_bi,i)=xrshg2*x(1)*dexp(-xcorsurf*x(ipot))
!           somethiol=cesp(iesp_thiol_mono,i)+cesp(iesp_thiol_bi,i)
!           if(cesp(iesp_thiol_mono,i) > cesp_max)then
!               cesp_max= cesp(iesp_thiol_mono,i)
!               iesp_max=iesp_thiol_mono
!           endif
!           if(cesp(iesp_thiol_bi,i) > cesp_max)then
!               cesp_max= cesp(iesp_thiol_bi,i)
!               iesp_max=iesp_thiol_bi
!           endif
!         endif

!  calcul des concentrations des especes particulaires (somme des sites)
!   on cherche [S-OC+] pour chaque substrat et chaque site
!    [S-OC+]=[S-OH]*Km*[C2+]*g2/actHplus(i)
! et pour site type 3 (thiol), on cherche aussi le site bidentate
!----------------------------------------------------------------
          cptot=somet+somethiol
          catpart=0.
 	  ik=0      
	  do jsc=1,nsubc
            js=jsubs(jsc)
            cesp(iespp_moco(js),i)=zero
            socat(iespp_moco(js),i)=zero
            do isit=1,nsit(js)
	      ik=ik+1
              isoh=1+nsubsic(jsc)+isit
              cesp(iespp_moco(js),i)=cesp(iespp_moco(js),i)+  &
                       x(isoh)*xikm(ik)*x(1)*xg2shp*xcsub(js)
              if(cesp(iespp_moco(js),i) > cesp_max)then
                 cesp_max= cesp(iespp_moco(js),i)
                 iesp_max=iespp_moco(js)
              endif

!               cation adsorbe sur substrat (nmole/l)
              if(ncomp==1)then
	        ipot=1+nsubsitc+js
                socat(iespp_moco(js),i)=socat(iespp_moco(js),i)+     &
                          x(isoh)*xgcat*xequcs(isit,js)*   &
                               dexp(-xcorsurf*x(ipot)*idcharc(isit,js))*xcsub(js)
              endif
            enddo
            cptot=cptot+cesp(iespp_moco(js),i)
            catpart=catpart+socat(iespp_moco(js),i)
          enddo
      endif
   
 ! reajustement des concentrations pour si il y a une perte trs faible
      difc=cptot+cdiss-c_conta_ech(i)
      if(abs(difc).lt.1.e-6)then
        cesp(iesp_max,i)=cesp(iesp_max,i)-difc
      else if(abs(difc).gt.1.e-6)then
         write(*,*)'ATTENTION MOCO non conservatif ??'
         write(*,*)'difference=',difc
      endif
      
      do in=1,nequtc
         if(abs(fver(in)).gt.1.d-3)then
            write(*,*)'ATTENTION'
            write(*,*)'il y a un probleme de convergence'
            write(*,*)'re-essayez avec d autres estimes initiaux'
            write(*,*)'estimes initiaux actuels :',xinit(1:nequtc)
            write(*,*)'x actuels :',x(1:nequtc)
            write(*,*)'equations=0 ',fver(1:nequtc)
            stop
          endif
       enddo 
  enddo

! ****************************************************************************
  END SUBROUTINE MOCO
  
!**************************************************************
      SUBROUTINE fcn(m,n,x,fvec,fjac,ldfjac,iflag)
!        ecriture des equations resolues par levenberg
!================================================================

   !! Arguments
integer,INTENT(IN)            :: m,n,ldfjac,iflag
real(kind=rlg),dimension(n),INTENT(INOUT) :: x,fvec
real(kind=rlg),dimension(ldfjac,n),INTENT(INOUT):: fjac

   !! * Local declarations
integer                :: jsc,neq0,ipot,isit,js,isoh,ieqa,ieqc
real(kind=rlg)         :: som1t,a1,a2,som,somt,qw,a3,a4,xxikm,xcikm,xtikm
real(kind=rlg)         :: as3,as4,som1,som2,som3,som2t,as1,as2,bbid
      
!	la derniere equation (conservation de la masse) est mise en premier

if (iflag==1) then

  som1t=zero
  do jsc=1,nsubc
     neq0=nequsc(jsc)*(jsc-1)+1
     ipot=1+nsubsitc+jsc
     js=jsubs(jsc)
     qw=-xcorsurf*x(ipot)
     som=zero
     somt=zero
     do isit=1,nsitc(jsc)
        isoh=1+nsubsic(jsc)+isit
        ieqa=neq0+isit
!          a1 = dixph/Ka1 ss dim
         a1=ika1(isit,js)*dixph/(xka1(isit,jsc)*dexp(idchars(isit,js,1)*qw))
!          a2 = Ka2/dixph ss dim
         a2=xka2(isit,jsc)*dexp(idchars(isit,js,2)*qw)/dixph
!          a3 = dixph/xactCd/cd2+/K13 ou 14  ==>>xg2shp en nmole/l ==>> ss dim
         a3=xg2shp*x(1)*xequms(isit,js)*dexp(idcharm(isit,js)*qw)

!          a4 = idem a a3 pour complexation du cation competiteur
!             =xactcat*ccat*Kcat/dixph/xact1
         a4=xgcat*xequcs(isit,js)*dexp(idcharc(isit,js)*qw)
!         somme des especes de surface chargees en nmole/l
         som=som+x(isoh)*(xchasu(jsc)*a3+xchasuc(jsc)*a4+a1-a2)

!          une equation par site et par substrat
!              (bilan de nombres de sites) somme en nmole/l
        fvec(ieqa)=-xsitac(isit,jsc)+x(isoh)*xcsub(js)*(1+a3+a4+a1+a2) 
!         write(*,*)'fvec,ieqa',ieqa,fvec(ieqa), xsitac(isit,jsc)
!         bbid=x(isoh)*xcsub(js)*(1+a3+a4+a1+a2)
!         write(*,*)xsitac(isit,jsc),x(isoh),xcsub(js),bbid

!           somme des especes pour conservation metal total
        somt=somt+x(isoh)*a3
     enddo

!      une equation par substrat : charge de surface  en nmole/l
     ieqc=neq0+nsitc(jsc)+1
     fvec(ieqc)=som*xrfsub(jsc)-x23i*x(ipot)
!!!     if(l_thiolpart .and. js==isub_thiolpart)fvec(ieqc)=fvec(ieqc)+  &
!!!             (crsmoins+(xrshg+xrshg2)*x(1)*dexp(-xcorsurf*x(ipot)))*xrfsub(jsc)/csub(js)
     som1t=som1t+somt*xcsub(js)
!     write(*,*)qw,a1,a2,a3,a4,som,somt,som1t,js,xcsub(js),fvec(ieqa)
  enddo

! derniere equation (bilan de masse) somme de nmole/l 
  fvec(1)=x(1)*(xsom+xsokcat)-xctot+som1t
!!!  if(l_thiolpart)fvec(1)=fvec(1)+(xrshg+xrshg2)*x(1)*dexp(-xcorsurf*x(ipot))
  
else if (iflag==2) then
!***
!* jacobienne
!***
  som2t=0.d0
!!!!! boucle sur les substrats
  do jsc=1,nsubc
    neq0=nequsc(jsc)*(jsc-1)+1
    ipot=1+nsubsitc+jsc
    js=jsubs(jsc)
    qw=-xcorsurf*x(ipot)
    som1=0.d0
    som2=0.d0
    som3=0.d0
    som1t=0.d0
    ieqc=neq0+nsitc(jsc)+1

!!!!! boucle sur les sites
    do isit=1,nsitc(jsc)
       isoh=1+nsubsic(jsc)+isit
!        	ikm=1+nsubsitc+nsubsic(jsc)+isit
       ieqa=neq0+isit
!        	ieqb=ieqa+nsitc(jsc)
       xxikm=xequms(isit,js)*dexp(idcharm(isit,js)*qw)
       a1=ika1(isit,js)*dixph/(xka1(isit,jsc)*dexp(idchars(isit,js,1)*qw))
! corrige avril 2010        as1=-a1/(xcorsurf*idchars(isit,js,1))
       as1=a1*xcorsurf*idchars(isit,js,1)
       a2=xka2(isit,jsc)*dexp(idchars(isit,js,2)*qw)/dixph
       as2=-a2*xcorsurf*idchars(isit,js,2)
       a3=xg2shp*x(1)*xxikm
       xcikm=xequcs(isit,js)*dexp(idcharc(isit,js)*qw)
       a4=xgcat*xcikm
       		
!         remplissage des lignes de la jacob
       fjac(ieqa,1:m)=zero
       fjac(ieqc,1:m)=zero
!        derivee par rapport a x(1)
       fjac(ieqa,1)=x(isoh)*xcsub(js)*xg2shp*xxikm

! derivee par rapport a x(isoh)   (is,js)
       fjac(ieqa,isoh)=xcsub(js)*(1+a1+a2+a3+a4)
       fjac(ieqc,isoh)=(a1-a2+a3*xchasu(jsc)+a4*xchasuc(jsc))*xrfsub(jsc)

! derivee par rapport a x(ipot)
       as3=-a3*xcorsurf*idcharm(isit,js)
       as4=-a4*xcorsurf*idcharc(isit,js)
       fjac(ieqa,ipot)=x(isoh)*xcsub(js)*(as1+as2+as3+as4)

       som1=xxikm*x(isoh)+som1
       som2=x(isoh)*(as1-as2+xchasu(jsc)*as3+xchasuc(jsc)*as4)+som2
       som3=x(isoh)*as3+som3
       som1t=som1t+x(isoh)*xxikm
       fjac(1,isoh)=xcsub(js)*a3

!    fin de la boucle sur les sites
    enddo

!    somme sur les substrat pour la derniere eq
    som2t=som2t+som1t*xcsub(js)
    fjac(ieqc,1)=som1*xrfsub(jsc)*xg2shp*xchasu(jsc)
!!!    if(l_thiolpart .and. js==isub_thiolpart)fjac(ieqc,1)=fjac(ieqc,1)+  &
!!!             (xrshg+xrshg2)*dexp(-xcorsurf*x(ipot))*xrfsub(jsc)/csub(js)
    fjac(ieqc,ipot)=som2*xrfsub(jsc)-x23i
!!!    if(l_thiolpart .and. js==isub_thiolpart)fjac(ieqc,ipot)=fjac(ieqc,ipot)  &
!!!          -x(1)*xcorsurf*(xrshg+xrshg2)*dexp(-xcorsurf*x(ipot))*xrfsub(jsc)/csub(js)
! je ne comprends pas cette version (ancien moco) 
!   fjac(1,ipot)=som3*xcsub(js)*x(1)*xg2shp
! je corrige (fevrier 2010 ??)
    fjac(1,ipot)=som3*xcsub(js)
!!!    if(l_thiolpart)fjac(1,ipot)=fjac(1,ipot) &
!!!          -x(1)*xcorsurf*(xrshg+xrshg2)*dexp(-xcorsurf*x(ipot))   
!     write(*,*)'qw..',qw,a1,a2,as1,as2
!     write(*,*)'xxikm..',xxikm,a3,xcikm,as3,as4,som1,som2,som3,som1t,som2t
!     write(*,*)'1',fjac(1,1:m)
!     write(*,*)'ieqa',ieqa,fjac(ieqa,1:m)
!     write(*,*)'ieqc',ieqc,fjac(ieqc,1:m)
!     write(*,*)isoh,ipot
     
  enddo

  fjac(1,1)=xsom+xsokcat+som2t*xg2shp
!!!  if(l_thiolpart) fjac(1,1)=fjac(1,1)+(xrshg+xrshg2)*dexp(-xcorsurf*x(ipot)) 

endif
!write(*,*)'fcn',iflag
!write(*,*)'x',x
!write(*,*)'fvec',fvec
!write(*,*)'fjac'
!do jsc=1,ldfjac
!do isit=1,n
!   write(*,*)jsc,isit,fjac(jsc,isit)
!enddo
!enddo

   END SUBROUTINE fcn
        
!**************************************************************
      SUBROUTINE verif(n,x,fver)
!        VERIFICATION des equations resolues par levenberg
!================================================================

   !! Arguments
integer,INTENT(IN)            :: n
real(kind=rlg),dimension(n),INTENT(OUT) :: fver
real(kind=rlg),dimension(n),INTENT(IN) :: x

   !! * Local declarations
integer                :: jsc,neq0,ipot,js,isoh,ieqa,ieqc,isit,ikm
real(kind=rlg)         :: som1t,qw,a1,a2,som,somt,a3,a4	

som1t=zero
do jsc=1,nsubc
  neq0=nequsc(jsc)*(jsc-1)+1
  ipot=1+nsubsitc+jsc
  js=jsubs(jsc)
  qw=-xcorsurf*x(ipot)
  som=0.d0
  somt=0.d0
  do isit=1,nsitc(jsc)
     isoh=1+nsubsic(jsc)+isit
     ieqa=neq0+isit
!       a1 = dixph/Ka1 ss dim
         a1=ika1(isit,js)*dixph/(xka1(isit,jsc)*dexp(idchars(isit,js,1)*qw))
!       a2 = Ka2/dixph ss dim
         a2=xka2(isit,jsc)*dexp(idchars(isit,js,2)*qw)/dixph
!       a3 = dixph/xactCd/cd2+/K13 ou 14  ==>>xg2shp en nmole/l ==>> ss dim
!        a3=xg2shp*x(1)*x(ikm)
     a3=xg2shp*x(1)*xequms(isit,js)*dexp(idcharm(isit,js)*qw)
!       a4 = idem a a3 pour complexation du cation competiteur
!          =xactcat*ccat*Kcat/dixph/xact1
     a4=xgcat*xequcs(isit,js)*dexp(idcharc(isit,js)*qw)

!        somme des especes de surface chargees en nmole/l
     som=som+x(isoh)*(xchasu(jsc)*a3+xchasuc(jsc)*a4+a1-a2)

!        une equation par site et par substrat
!            (bilan de nombres de sites) somme en nmole/l
     fver(ieqa)=-xsitac(isit,jsc)+x(isoh)*xcsub(js)*(1+a3+a4+a1+a2)

!       somme des especes pour conservation metal total
     somt=somt+x(isoh)*a3
  enddo

!       une equation par substrat : charge de surface  en nmole/l
  ieqc=neq0+nsitc(jsc)+1
  fver(ieqc)=som*xrfsub(jsc)-x23i*x(ipot)
  som1t=som1t+somt*xcsub(js)

enddo

!       derniere equation (bilan de masse) somme de nmole/l 
  fver(1)=x(1)*(xsom+xsokcat)-xctot+som1t

   END SUBROUTINE verif
!****************************************************************
   SUBROUTINE lmpar(n,r,ldr,ipvt,diag,qtb,delta,par,x,sdiag,wa1,wa2)

!  given an m by n matrix a, an n by n nonsingular diagonal
!  matrix d, an m-vector b, and a positive number delta,
!  the problem is to determine a value for the parameter
!  par such that if x solves the system
!        a*x = b ,     sqrt(par)*d*x = 0 ,
!  in the least squares sense, and dxnorm is the euclidean
!  norm of d*x, then either par is zero and
!        (dxnorm-delta) .le. 0.1*delta ,
!  or par is positive and
!        abs(dxnorm-delta) .le. 0.1*delta .

!  this subroutine completes the solution of the problem
!  if it is provided with the necessary information from the
!  qr factorization, with column pivoting, of a. that is, if
!  a*p = q*r, where p is a permutation matrix, q has orthogonal
!  columns, and r is an upper triangular matrix with diagonal
!  elements of nonincreasing magnitude, then lmpar expects
!  the full upper triangle of r, the permutation matrix p,
!  and the first n components of (q transpose)*b. on output
!  lmpar also provides an upper triangular matrix s such that
!         t   t                   t
!        p *(a *a + par*d*d)*p = s *s .

!  s is employed within lmpar and may be of separate interest.

!  only a few iterations are generally needed for convergence
!  of the algorithm. if, however, the limit of 10 iterations
!  is reached, then the output par will contain the best
!  value obtained so far.

!  the subroutine statement is
!    subroutine lmpar(n,r,ldr,ipvt,diag,qtb,delta,par,x,sdiag,
!                     wa1,wa2)
!  where
!    n is a positive integer input variable set to the order of r.
!    r is an n by n array. on input the full upper triangle
!      must contain the full upper triangle of the matrix r.
!      on output the full upper triangle is unaltered, and the
!      strict lower triangle contains the strict upper triangle
!      (transposed) of the upper triangular matrix s.
!    ldr is a positive integer input variable not less than n
!      which specifies the leading dimension of the array r.
!    ipvt is an integer input array of length n which defines the
!      permutation matrix p such that a*p = q*r. column j of p
!      is column ipvt(j) of the identity matrix.
!    diag is an input array of length n which must contain the
!      diagonal elements of the matrix d.
!    qtb is an input array of length n which must contain the first
!      n elements of the vector (q transpose)*b.
!    delta is a positive input variable which specifies an upper
!      bound on the euclidean norm of d*x.
!    par is a nonnegative variable. on input par contains an
!      initial estimate of the levenberg-marquardt parameter.
!      on output par contains the final estimate
!    x is an output array of length n which contains the least
!      squares solution of the system a*x = b, sqrt(par)*d*x = 0,
!      for the output par.
!    sdiag is an output array of length n which contains the
!      diagonal elements of the upper triangular matrix s.
!    wa1 and wa2 are work arrays of length n.
!  subprograms called
!    minpack-supplied ... dpmpar,enorm,qrsolv
!    fortran-supplied ... dabs,dmax1,dmin1,dsqrt

!  argonne national laboratory. minpack project. march 1980.
!  burton s. garbow, kenneth e. hillstrom, jorge j. more

!  **********
   !! Arguments
integer,INTENT(IN)            :: n,ldr
integer,dimension(n),INTENT(IN):: ipvt
real(kind=rlg),INTENT(IN)     :: delta
real(kind=rlg),INTENT(INOUT)     :: par
real(kind=rlg),dimension(n),INTENT(IN) :: diag,qtb
real(kind=rlg),dimension(n),INTENT(OUT) :: x,sdiag
real(kind=rlg),dimension(n)             :: wa1,wa2
real(kind=rlg),dimension(ldr,n),INTENT(INOUT):: r

   !! * Local declarations
integer                :: i,iter,j,jm1,jp1,k,l,nsing
real(kind=rlg)         :: dxnorm,dwarf,fp,gnorm,parc,parl,paru
real(kind=rlg)         :: suml,temp
      
!  dwarf is the smallest positive magnitude.
!dwarf = dpmpar(2)
dwarf=tiny(0._rlg)
j=kind(1.d00)

!  compute and store in x the gauss-newton direction. if the
!  jacobian is rank-deficient, obtain a least squares solution.
nsing = n
do j = 1, n
   wa1(j) = qtb(j)
   if (r(j,j) .eq. zero .and. nsing .eq. n) nsing = j - 1
   if (nsing .lt. n) wa1(j) = zero
enddo
!write(*,*)'dans lmpar wa1',wa1
if (nsing .ge. 1) then
  do k = 1, nsing
     j = nsing - k + 1
     wa1(j) = wa1(j)/r(j,j)
!     write(*,*)k,j,wa1(j),r(j,j)
     temp = wa1(j)
     jm1 = j - 1
     if (jm1 .ge. 1) then
       do i=1,jm1
!        write(*,*)k,j,i,wa1(i),r(i,j),temp
        wa1(i) = wa1(i) - r(i,j)*temp
!        write(*,*)k,j,i,wa1(i),r(i,j),temp
       enddo
     endif
  enddo
endif
!write(*,*)'dans LMPAR wa1 2',wa1
do j = 1, n
   l = ipvt(j)
   x(l) = wa1(j)
enddo

!  initialize the iteration counter.
!  evaluate the function at the origin, and test
!  for acceptance of the gauss-newton direction.

iter = 0
wa2(1:n) = diag(1:n)*x(1:n)
dxnorm = enorm(n,wa2)
fp = dxnorm - delta
!write(*,*)'dans lmpar fp',fp,p1,delta
if (fp .gt. p1*delta) then

!  if the jacobian is not rank deficient, the newton
!  step provides a lower bound, parl, for the zero of
!  the function. otherwise set this bound to zero.

  parl = zero
  if (nsing .ge. n) then
     do j = 1, n
       l = ipvt(j)
       wa1(j) = diag(l)*(wa2(l)/dxnorm)
     enddo
!write(*,*)'dans lmpar wa1 3',wa1
     do j = 1, n
        suml = zero
        jm1 = j - 1
        do i=1,jm1
          if (jm1 .ge. 1) suml=suml+r(i,j)*wa1(i)
        enddo
        wa1(j) = (wa1(j) - suml  )/r(j,j)
     enddo 
!write(*,*)'dans lmpar wa1 4',wa1
     temp = enorm(n,wa1)
     parl = ((fp/delta)/temp)/temp
  endif

!  calculate an upper bound, paru, for the zero of the function.
  do j = 1, n
     suml = zero
     do i = 1, j
        suml= suml+ r(i,j)*qtb(i)
     enddo
     l = ipvt(j)
     wa1(j) = suml/diag(l)
  enddo
!write(*,*)'dans lmpar wa1 5',wa1
  gnorm = enorm(n,wa1)
  paru = gnorm/delta
  if (paru == zero) paru = dwarf/dmin1(delta,p1)

!  if the input par lies outside of the interval (parl,paru),
!  set par to the closer endpoint.
  par = dmax1(par,parl)
  par = dmin1(par,paru)
  if (par == zero) par = gnorm/dxnorm

!  beginning of an iteration.
  do while(iter < 10)
    iter = iter + 1

!     evaluate the function at the current value of par.
    if (par == zero) par = dmax1(dwarf,p001*paru)
    temp = dsqrt(par)
    wa1(1:n) = temp*diag(1:n)
!    write(*,*)'avant qrsolv',iter,wa1
    call qrsolv(n,r,ldr,ipvt,wa1,qtb,x,sdiag,wa2)
!    write(*,*)'apres qrsolv',wa1
    wa2(1:n) = diag(1:n)*x(1:n)
    dxnorm = enorm(n,wa2)
    temp = fp
    fp = dxnorm - delta

!     if the function is small enough, accept the current value
!     of par. also test for the exceptional cases where parl
!     is zero or the number of iterations has reached 10.
    if (dabs(fp) .le. p1*delta.or. parl .eq. zero .and. fp .le. temp  &
                 .and. temp .lt. zero ) exit

!     compute the newton correction.
    do j = 1, n
        l = ipvt(j)
        wa1(j) = diag(l)*(wa2(l)/dxnorm)
    enddo
!    write(*,*)'iter 2',wa1
    do j = 1, n
       wa1(j) = wa1(j)/sdiag(j)
       temp = wa1(j)
       jp1 = j + 1
       if (n .ge. jp1) then
         wa1(jp1:n) = wa1(jp1:n) - r(jp1:n,j)*temp
       endif
    enddo
!    write(*,*)'iter 3',wa1
    temp = enorm(n,wa1)
    parc = ((fp/delta)/temp)/temp

!     depending on the sign of the function, update parl or paru.
    if (fp > zero) parl = dmax1(parl,par)
    if (fp < zero) paru = dmin1(paru,par)

!     compute an improved estimate for par.
    par = dmax1(parl,par+parc)

!     end of an iteration.
  enddo
endif

!  termination.
if (iter == 0) par = zero

   END SUBROUTINE lmpar

!*************************************************************
      FUNCTION enorm(nn,x)
!  **********
!  given an nn-vector x, this function calculates the
!  euclidean norm of x.
!  the euclidean norm is computed by accumulating the sum of
!  squares in three different sums. the sums of squares for the
!  small and large components are scaled so that no overflows
!  occur. non-destructive underflows are permitted. underflows
!  and overflows do not occur in the computation of the unscaled
!  sum of squares for the intermediate components.
!  the definitions of small, intermediate and large components
!  depend on two constants, rdwarf and rgiant. the main
!  restrictions on these constants are that rdwarf**2 not
!  underflow and rgiant**2 not overflow. the constants
!  given here are suitable for every known computer.

!  the function statement is

!    function enorm(n,x)
!  where
!    n is a positive integer input variable.
!    x is an input array of length n.

!  subprograms called
!    fortran-supplied ... dabs,dsqrt

!  argonne national laboratory. minpack project. march 1980.
!  burton s. garbow, kenneth e. hillstrom, jorge j. more
!  **********
   !! Arguments
integer,INTENT(IN)            :: nn
real(kind=rlg),dimension(nn),INTENT(IN) :: x
real(kind=rlg)                :: enorm
   !! * Local declarations
integer                       :: i
real(kind=rlg)                :: agiant,floatn
real(kind=rlg)                :: s1,s2,s3,xabs,x1max,x3max

s1 = zero
s2 = zero
s3 = zero
x1max = zero
x3max = zero
floatn = nn
agiant = rgiant/floatn
do i = 1, nn
  xabs = dabs(x(i))
  if (xabs .le. rdwarf .or. xabs .ge. agiant) then
     if (xabs .gt. rdwarf) then

!           sum for large components.
        if (xabs .gt. x1max) then
           s1 = one + s1*(x1max/xabs)**2
           x1max = xabs
        else
           s1 = s1 + (xabs/x1max)**2
        endif
     else

!           sum for small components.
        if (xabs .gt. x3max) then
           s3 = one + s3*(x3max/xabs)**2
           x3max = xabs
        else
           if (xabs .ne. zero) s3 = s3 + (xabs/x3max)**2
        endif
     endif
   else

!        sum for intermediate components.
     s2 = s2 + xabs**2
   endif
enddo

!  calculation of norm.
if (s1 .ne. zero) then
   enorm = x1max*dsqrt(s1+(s2/x1max)/x1max)
else
   if (s2 .ne. zero) then
     if (s2 .ge. x3max)then
         enorm = dsqrt(s2*(one+(x3max/s2)*(x3max*s3)))
     else
         enorm = dsqrt(x3max*((s2/x3max)+(x3max*s3)))
     endif
  else
     enorm = x3max*dsqrt(s3)
  endif
endif
END FUNCTION enorm

!*************************************************************
      SUBROUTINE qrfac(m,n,a,lda,pivot,ipvt,lipvt,rdiag,acnorm,wa)
!  **********
!  this subroutine uses householder transformations with column
!  pivoting (optional) to compute a qr factorization of the
!  m by n matrix a. that is, qrfac determines an orthogonal
!  matrix q, a permutation matrix p, and an upper trapezoidal
!  matrix r with diagonal elements of nonincreasing magnitude,
!  such that a*p = q*r. the householder transformation for
!  column k, k = 1,2,...,min(m,n), is of the form
!                        t
!        i - (1/u(k))*u*u
!  where u has zeros in the first k-1 positions. the form of
!  this transformation and the method of pivoting first
!  appeared in the corresponding linpack subroutine.

!  the subroutine statement is

!    subroutine qrfac(m,n,a,lda,pivot,ipvt,lipvt,rdiag,acnorm,wa)
!  where
!    m is a positive integer input variable set to the number
!      of rows of a.
!    n is a positive integer input variable set to the number
!      of columns of a.
!    a is an m by n array. on input a contains the matrix for
!      which the qr factorization is to be computed. on output
!      the strict upper trapezoidal part of a contains the strict
!      upper trapezoidal part of r, and the lower trapezoidal
!      part of a contains a factored form of q (the non-trivial
!      elements of the u vectors described above).
!    lda is a positive integer input variable not less than m
!      which specifies the leading dimension of the array a.
!    pivot is a logical input variable. if pivot is set true,
!      then column pivoting is enforced. if pivot is set false,
!      then no column pivoting is done.
!    ipvt is an integer output array of length lipvt. ipvt
!      defines the permutation matrix p such that a*p = q*r.
!      column j of p is column ipvt(j) of the identity matrix.
!      if pivot is false, ipvt is not referenced.
!    lipvt is a positive integer input variable. if pivot is false,
!      then lipvt may be as small as 1. if pivot is true, then
!      lipvt must be at least n.
!    rdiag is an output array of length n which contains the
!      diagonal elements of r.
!    acnorm is an output array of length n which contains the
!      norms of the corresponding columns of the input matrix a.
!      if this information is not needed, then acnorm can coincide
!      with rdiag.
!    wa is a work array of length n. if pivot is false, then wa
!      can coincide with rdiag.

!  subprograms called
!    minpack-supplied ... dpmpar,enorm
!    fortran-supplied ... dmax1,dsqrt,min0

!  argonne national laboratory. minpack project. march 1980.
!  burton s. garbow, kenneth e. hillstrom, jorge j. more
!  **********
  !! Arguments
integer,INTENT(IN)            :: m,n,lda,lipvt
logical,INTENT(IN)            :: pivot
real(kind=rlg),dimension(n)             :: wa
real(kind=rlg),dimension(lda,n),INTENT(INOUT):: a
integer,INTENT(OUT),dimension(lipvt)     :: ipvt
real(kind=rlg),dimension(n),INTENT(OUT) :: rdiag,acnorm

   !! * Local declarations
integer                :: i,j,jp1,k,kmax,minmn
real(kind=rlg)         :: ajnorm,epsmch,suml,temp
 
!  epsmch is the machine precision.
!epsmch = dpmpar(1)
epsmch =epsilon(0._rlg)

!  compute the initial column norms and initialize several arrays.
do j = 1,n
   acnorm(j) = enorm(m,a(1,j))
   rdiag(j) = acnorm(j)
   wa(j) = rdiag(j)
   if (pivot) ipvt(j) = j
enddo

!  reduce a to r with householder transformations.
minmn = min0(m,n)
do j = 1, minmn
   if (pivot) then

!     bring the column of largest norm into the pivot position.
      kmax = j
      do k = j, n
        if (rdiag(k) .gt. rdiag(kmax)) kmax = k
      enddo
      if (kmax .ne. j) then
         do i = 1, m
            temp = a(i,j)
            a(i,j) = a(i,kmax)
            a(i,kmax) = temp
         enddo
         rdiag(kmax) = rdiag(j)
         wa(kmax) = wa(j)
         k = ipvt(j)
         ipvt(j) = ipvt(kmax)
         ipvt(kmax) = k
      endif
   endif
!   write(*,*)'j',j,kmax,rdiag
!     compute the householder transformation to reduce the
!     j-th column of a to a multiple of the j-th unit vector.
   ajnorm = enorm(m-j+1,a(j,j))
!   write(*,*)'ajnorm',ajnorm
   
   if (ajnorm .ne. zero) then
      if (a(j,j) .lt. zero) ajnorm = -ajnorm
      a(j:m,j) = a(j:m,j)/ajnorm
      a(j,j) = a(j,j) + one

!         apply the transformation to the remaining columns
!          and update the norms.
      jp1 = j + 1
      if (n .ge. jp1) then
        do k = jp1, n
           suml   = zero
           do i = j, m
              suml   = suml   + a(i,j)*a(i,k)
           enddo
           temp = suml  /a(j,j)
!           write(*,*)j,k,suml
!           write(*,*)'a',a
           
           a(j:m,k) = a(j:m,k) - temp*a(j:m,j)
!           write(*,*)'a2',a
           if (pivot .and. rdiag(k) .ne. zero) then
              temp = a(j,k)/rdiag(k)
!              write(*,*)j,k,rdiag(k)
              rdiag(k) = rdiag(k)*dsqrt(dmax1(zero,one-temp**2))
!              write(*,*)j,k,rdiag(k)
              if (p05*(rdiag(k)/wa(k))**2 .le. epsmch) then
                  rdiag(k) = enorm(m-j,a(jp1,k))
                  wa(k) = rdiag(k)
!                  write(*,*)'p05',j,k,rdiag(k),wa(k)
              endif
           endif
        enddo
      endif
   endif
   rdiag(j) = -ajnorm
enddo
!write(*,*)'fin qrfac acnorm=',acnorm
!write(*,*)'fin qrfac rdiag=',rdiag

   END SUBROUTINE qrfac

!!!**************************************************************
      SUBROUTINE qrsolv(n,r,ldr,ipvt,diag,qtb,x,sdiag,wa)
!  **********
!  given an m by n matrix a, an n by n diagonal matrix d,
!  and an m-vector b, the problem is to determine an x which
!  solves the system
!        a*x = b ,     d*x = 0 ,
!  in the least squares sense.

!  this subroutine completes the solution of the problem
!  if it is provided with the necessary information from the
!  qr factorization, with column pivoting, of a. that is, if
!  a*p = q*r, where p is a permutation matrix, q has orthogonal
!  columns, and r is an upper triangular matrix with diagonal
!  elements of nonincreasing magnitude, then qrsolv expects
!  the full upper triangle of r, the permutation matrix p,
!  and the first n components of (q transpose)*b. the system
!  a*x = b, d*x = 0, is then equivalent to
!               t       t
!        r*z = q *b ,  p *d*p*z = 0 ,

!  where x = p*z. if this system does not have full rank,
!  then a least squares solution is obtained. on output qrsolv
!  also provides an upper triangular matrix s such that

!         t   t               t
!        p *(a *a + d*d)*p = s *s .

!  s is computed within qrsolv and may be of separate interest.

!  the subroutine statement is
!    subroutine qrsolv(n,r,ldr,ipvt,diag,qtb,x,sdiag,wa)
!  where
!    n is a positive integer input variable set to the order of r.
!    r is an n by n array. on input the full upper triangle
!      must contain the full upper triangle of the matrix r.
!      on output the full upper triangle is unaltered, and the
!      strict lower triangle contains the strict upper triangle
!      (transposed) of the upper triangular matrix s.
!    ldr is a positive integer input variable not less than n
!      which specifies the leading dimension of the array r.
!    ipvt is an integer input array of length n which defines the
!      permutation matrix p such that a*p = q*r. column j of p
!      is column ipvt(j) of the identity matrix.
!    diag is an input array of length n which must contain the
!      diagonal elements of the matrix d.
!    qtb is an input array of length n which must contain the first
!      n elements of the vector (q transpose)*b.
!    x is an output array of length n which contains the least
!      squares solution of the system a*x = b, d*x = 0.
!    sdiag is an output array of length n which contains the
!      diagonal elements of the upper triangular matrix s.
!    wa is a work array of length n.

!  subprograms called
!    fortran-supplied ... dabs,dsqrt

!  argonne national laboratory. minpack project. march 1980.
!  burton s. garbow, kenneth e. hillstrom, jorge j. more
!  **********
  !! Arguments
integer,INTENT(IN)            :: n,ldr
integer,INTENT(IN),dimension(n)     :: ipvt
real(kind=rlg),dimension(n)             :: wa,diag,qtb
real(kind=rlg),dimension(ldr,n),INTENT(INOUT):: r 
real(kind=rlg),dimension(n),INTENT(OUT) :: x,sdiag

   !! * Local declarations
integer                :: i,j,jp1,k,kp1,l,nsing
real(kind=rlg)         :: cos,cotan,qtbpj,sin,suml,tan,temp
 
!  copy r and (q transpose)*b to preserve input and initialize s.
!  in particular, save the diagonal elements of r in x.
do j = 1, n
  do i = j, n
    r(i,j) = r(j,i)
  enddo
  x(j) = r(j,j)
  wa(j) = qtb(j)
enddo

!  eliminate the diagonal matrix d using a givens rotation.
do j = 1, n

!     prepare the row of d to be eliminated, locating the
!     diagonal element using p from the qr factorization.
  l = ipvt(j)
  if (diag(l) .ne. zero) then
    sdiag(j:n) = zero
    sdiag(j) = diag(l)

!     the transformations to eliminate the row of d
!     modify only a single element of (q transpose)*b
!     beyond the first n, which is initially zero.
    qtbpj = zero
    do k = j, n

!        determine a givens rotation which eliminates the
!        appropriate element in the current row of d.
      if (sdiag(k) .ne. zero) then
         if (dabs(r(k,k)) .lt. dabs(sdiag(k))) then
            cotan = r(k,k)/sdiag(k)
            sin = p5/dsqrt(p25+p25*cotan**2)
            cos = sin*cotan
         else
            tan = sdiag(k)/r(k,k)
            cos = p5/dsqrt(p25+p25*tan**2)
            sin = cos*tan
         endif

!           compute the modified diagonal element of r and
!           the modified element of ((q transpose)*b,0).
         r(k,k) = cos*r(k,k) + sin*sdiag(k)
         temp = cos*wa(k) + sin*qtbpj
         qtbpj = -sin*wa(k) + cos*qtbpj
         wa(k) = temp

!           accumulate the tranformation in the row of s.
         kp1 = k + 1
         if (n .ge. kp1) then
            do i = kp1,n
               temp = cos*r(i,k) + sin*sdiag(i)
               sdiag(i) = -sin*r(i,k) + cos*sdiag(i)
               r(i,k) = temp
            enddo
         endif
      endif
    enddo
  endif

!     store the diagonal element of s and restore
!     the corresponding diagonal element of r.

  sdiag(j) = r(j,j)
  r(j,j) = x(j)
enddo

!  solve the triangular system for z. if the system is
!  singular, then obtain a least squares solution.
nsing = n
do j = 1, n
  if (sdiag(j) .eq. zero .and. nsing .eq. n) nsing = j - 1
  if (nsing .lt. n) wa(j) = zero
enddo
if (nsing .ge. 1) then
  do k = 1, nsing
      j = nsing - k + 1
      suml   = zero
      jp1 = j + 1
      if (nsing .ge. jp1) then
         do i = jp1, nsing
            suml   = suml   + r(i,j)*wa(i)
         enddo
      endif
      wa(j) = (wa(j) - suml  )/sdiag(j)
  enddo
endif

!  permute the components of z back to components of x.
do j = 1, n
  l = ipvt(j)
  x(l) = wa(j)
enddo

   END SUBROUTINE qrsolv

!!!!**************************************************************
      SUBROUTINE lmder1(m,n,x,fvec,fjac,ldfjac,info,ipvt,wa,lwa)
!  **********

!  the purpose of lmder1 is to minimize the sum of the squares of
!  m nonlinear functions in n variables by a modification of the
!  levenberg-marquardt algorithm. this is done by using the more
!  general least-squares solver lmder. the user must provide a
!  subroutine which calculates the functions and the jacobian.

!  the subroutine statement is
!    subroutine lmder1(fcn,m,n,x,fvec,fjac,ldfjac,tol,info,
!                      ipvt,wa,lwa)
!  where
!    fcn is the name of the user-supplied subroutine which
!      calculates the functions and the jacobian. fcn must
!      be declared in an external statement in the user
!      calling program, and should be written as follows.

!      subroutine fcn(m,n,x,fvec,fjac,ldfjac,iflag)
!      integer m,n,ldfjac,iflag
!      double precision x(n),fvec(m),fjac(ldfjac,n)
!      ----------
!      if iflag = 1 calculate the functions at x and
!      return this vector in fvec. do not alter fjac.
!      if iflag = 2 calculate the jacobian at x and
!      return this matrix in fjac. do not alter fvec.
!      ----------
!      return
!      end

!      the value of iflag should not be changed by fcn unless
!      the user wants to terminate execution of lmder1.
!      in this case set iflag to a negative integer.
!    m is a positive integer input variable set to the number
!      of functions.
!    n is a positive integer input variable set to the number
!      of variables. n must not exceed m.
!    x is an array of length n. on input x must contain
!      an initial estimate of the solution vector. on output x
!      contains the final estimate of the solution vector.
!    fvec is an output array of length m which contains
!      the functions evaluated at the output x.
!    fjac is an output m by n array. the upper n by n submatrix
!      of fjac contains an upper triangular matrix r with
!      diagonal elements of nonincreasing magnitude such that

!             t     t           t
!            p *(jac *jac)*p = r *r,

!      where p is a permutation matrix and jac is the final
!      calculated jacobian. column j of p is column ipvt(j)
!      (see below) of the identity matrix. the lower trapezoidal
!      part of fjac contains information generated during
!      the computation of r.
!    ldfjac is a positive integer input variable not less than m
!      which specifies the leading dimension of the array fjac.
!    tol is a nonnegative input variable. termination occurs
!      when the algorithm estimates either that the relative
!      error in the sum of squares is at most tol or that
!      the relative error between x and the solution is at
!      most tol.
!    info is an integer output variable. if the user has
!      terminated execution, info is set to the (negative)
!      value of iflag. see description of fcn. otherwise,
!      info is set as follows.
!      info = 0  improper input parameters.
!      info = 1  algorithm estimates that the relative error
!                in the sum of squares is at most tol.
!      info = 2  algorithm estimates that the relative error
!                between x and the solution is at most tol.
!      info = 3  conditions for info = 1 and info = 2 both hold.
!      info = 4  fvec is orthogonal to the columns of the
!                jacobian to machine precision.
!      info = 5  number of calls to fcn with iflag = 1 has
!                reached 100*(n+1).
!      info = 6  tol is too small. no further reduction in
!                the sum of squares is possible.
!      info = 7  tol is too small. no further improvement in
!                the approximate solution x is possible.
!    ipvt is an integer output array of length n. ipvt
!      defines a permutation matrix p such that jac*p = q*r,
!      where jac is the final calculated jacobian, q is
!      orthogonal (not stored), and r is upper triangular
!      with diagonal elements of nonincreasing magnitude.
!      column j of p is column ipvt(j) of the identity matrix.
!    wa is a work array of length lwa.
!    lwa is a positive integer input variable not less than 5*n+m.

!  subprograms called
!    user-supplied ...... fcn
!    minpack-supplied ... lmder

!  argonne national laboratory. minpack project. march 1980.
!  burton s. garbow, kenneth e. hillstrom, jorge j. more
!  **********
  !! Arguments
integer,INTENT(IN)                            :: m,n,ldfjac,lwa
integer,INTENT(OUT)                           :: info
integer,INTENT(OUT),dimension(n)              :: ipvt
real(kind=rlg),dimension(lwa)                 :: wa
real(kind=rlg),dimension(n),INTENT(INOUT)     :: x 
real(kind=rlg),dimension(m),INTENT(OUT)       :: fvec 
real(kind=rlg),dimension(ldfjac,n),INTENT(OUT):: fjac 

   !! * Local declarations
integer                :: maxfev,mode,nfev,njev,nprint
real(kind=rlg)         :: ftol,gtol,xtol

info = 0

!  check the input parameters for errors.
if (n .gt. 0 .and. m .ge. n .and. ldfjac .ge. m .and. tol .ge. zero   &
         .and. lwa .ge. (5*n + m)) then
      maxfev = 100*(n + 1)
      ftol = tol
      xtol = tol
      gtol = zero
      mode = 1
      nprint = 0
      call lmder(m,n,x,fvec,fjac,ldfjac,ftol,xtol,gtol,maxfev,   &
                wa(1),mode,factor,nprint,info,nfev,njev,ipvt,wa(n+1),   &
                wa(2*n+1),wa(3*n+1),wa(4*n+1),wa(5*n+1))
      if (info .eq. 8) info = 4
endif

   END SUBROUTINE lmder1

!**************************************************************
      SUBROUTINE lmder(m,n,x,fvec,fjac,ldfjac,ftol,xtol,gtol,   &
                       maxfev,diag,mode,factor,nprint,info,nfev,njev,    &
                      ipvt,qtf,wa1,wa2,wa3,wa4)
!  **********
!  subroutine lmder
!  the purpose of lmder is to minimize the sum of the squares of
!  m nonlinear functions in n variables by a modification of
!  the levenberg-marquardt algorithm. the user must provide a
!  subroutine which calculates the functions and the jacobian.

!  the subroutine statement is
!    subroutine lmder(fcn,m,n,x,fvec,fjac,ldfjac,ftol,xtol,gtol,
!                     maxfev,diag,mode,factor,nprint,info,nfev,
!                     njev,ipvt,qtf,wa1,wa2,wa3,wa4)
!  where
!    fcn is the name of the user-supplied subroutine which
!      calculates the functions and the jacobian. fcn must
!      be declared in an external statement in the user
!      calling program, and should be written as follows.

!      subroutine fcn(m,n,x,fvec,fjac,ldfjac,iflag)
!      integer m,n,ldfjac,iflag
!      double precision x(n),fvec(m),fjac(ldfjac,n)
!      ----------
!      if iflag = 1 calculate the functions at x and
!      return this vector in fvec. do not alter fjac.
!      if iflag = 2 calculate the jacobian at x and
!      return this matrix in fjac. do not alter fvec.
!      ----------
!      return
!      end

!      the value of iflag should not be changed by fcn unless
!      the user wants to terminate execution of lmder.
!      in this case set iflag to a negative integer.
!    m is a positive integer input variable set to the number
!      of functions.
!    n is a positive integer input variable set to the number
!      of variables. n must not exceed m.
!    x is an array of length n. on input x must contain
!      an initial estimate of the solution vector. on output x
!      contains the final estimate of the solution vector.
!    fvec is an output array of length m which contains
!      the functions evaluated at the output x.
!    fjac is an output m by n array. the upper n by n submatrix
!      of fjac contains an upper triangular matrix r with
!      diagonal elements of nonincreasing magnitude such that
!
!             t     t           t
!            p *(jac *jac)*p = r *r,
!
!      where p is a permutation matrix and jac is the final
!      calculated jacobian. column j of p is column ipvt(j)
!      (see below) of the identity matrix. the lower trapezoidal
!      part of fjac contains information generated during
!      the computation of r.
!    ldfjac is a positive integer input variable not less than m
!      which specifies the leading dimension of the array fjac.
!    ftol is a nonnegative input variable. termination
!      occurs when both the actual and predicted relative
!      reductions in the sum of squares are at most ftol.
!      therefore, ftol measures the relative error desired
!      in the sum of squares.
!    xtol is a nonnegative input variable. termination
!      occurs when the relative error between two consecutive
!      iterates is at most xtol. therefore, xtol measures the
!      relative error desired in the approximate solution.
!    gtol is a nonnegative input variable. termination
!      occurs when the cosine of the angle between fvec and
!      any column of the jacobian is at most gtol in absolute
!      value. therefore, gtol measures the orthogonality
!      desired between the function vector and the columns
!      of the jacobian.
!    maxfev is a positive integer input variable. termination
!      occurs when the number of calls to fcn with iflag = 1
!      has reached maxfev.
!    diag is an array of length n. if mode = 1 (see
!      below), diag is internally set. if mode = 2, diag
!      must contain positive entries that serve as
!      multiplicative scale factors for the variables.
!    mode is an integer input variable. if mode = 1, the
!      variables will be scaled internally. if mode = 2,
!      the scaling is specified by the input diag. other
!      values of mode are equivalent to mode = 1.
!    factor is a positive input variable used in determining the
!      initial step bound. this bound is set to the product of
!      factor and the euclidean norm of diag*x if nonzero, or else
!      to factor itself. in most cases factor should lie in the
!      interval (.1,100.).100. is a generally recommended value.
!    nprint is an integer input variable that enables controlled
!      printing of iterates if it is positive. in this case,
!      fcn is called with iflag = 0 at the beginning of the first
!      iteration and every nprint iterations thereafter and
!      immediately prior to return, with x, fvec, and fjac
!      available for printing. fvec and fjac should not be
!      altered. if nprint is not positive, no special calls
!      of fcn with iflag = 0 are made.
!    info is an integer output variable. if the user has
!      terminated execution, info is set to the (negative)
!      value of iflag. see description of fcn. otherwise,
!      info is set as follows.
!      info = 0  improper input parameters.
!      info = 1  both actual and predicted relative reductions
!                in the sum of squares are at most ftol.
!      info = 2  relative error between two consecutive iterates
!                is at most xtol.
!      info = 3  conditions for info = 1 and info = 2 both hold.
!      info = 4  the cosine of the angle between fvec and any
!                column of the jacobian is at most gtol in
!                absolute value.
!      info = 5  number of calls to fcn with iflag = 1 has
!                reached maxfev.
!      info = 6  ftol is too small. no further reduction in
!                the sum of squares is possible.
!      info = 7  xtol is too small. no further improvement in
!                the approximate solution x is possible.
!      info = 8  gtol is too small. fvec is orthogonal to the
!                columns of the jacobian to machine precision.
!    nfev is an integer output variable set to the number of
!      calls to fcn with iflag = 1.
!    njev is an integer output variable set to the number of
!      calls to fcn with iflag = 2.
!    ipvt is an integer output array of length n. ipvt
!      defines a permutation matrix p such that jac*p = q*r,
!      where jac is the final calculated jacobian, q is
!      orthogonal (not stored), and r is upper triangular
!      with diagonal elements of nonincreasing magnitude.
!      column j of p is column ipvt(j) of the identity matrix.
!    qtf is an output array of length n which contains
!      the first n elements of the vector (q transpose)*fvec.
!    wa1, wa2, and wa3 are work arrays of length n.
!    wa4 is a work array of length m.

!  subprograms called
!    user-supplied ...... fcn
!    minpack-supplied ... dpmpar,enorm,lmpar,qrfac
!    fortran-supplied ... dabs,dmax1,dmin1,dsqrt,mod

!  argonne national laboratory. minpack project. march 1980.
!  burton s. garbow, kenneth e. hillstrom, jorge j. more
!  **********
  !! Arguments
integer,INTENT(IN)                            :: m,n,ldfjac,maxfev,mode,nprint
integer,INTENT(OUT)                           :: info,nfev,njev
integer,INTENT(OUT),dimension(n)              :: ipvt
real(kind=rlg),INTENT(IN)                     :: ftol,xtol,gtol,factor
real(kind=rlg),dimension(n)                   :: diag,wa1,wa2,wa3
real(kind=rlg),dimension(m)                   :: wa4
real(kind=rlg),dimension(n),INTENT(INOUT)     ::  x
real(kind=rlg),dimension(m),INTENT(OUT)       ::  fvec
real(kind=rlg),dimension(n),INTENT(OUT)       ::  qtf
real(kind=rlg),dimension(ldfjac,n),INTENT(OUT)::  fjac

   !! * Local declarations
integer                :: i,iflag,iter,j,l,itest
real(kind=rlg)         :: actred,delta,dirder,epsmch,fnorm,fnorm1,gnorm,  &
                      par,pnorm,prered,ratio,suml,temp,temp1,temp2,xnorm  

!  epsmch is the machine precision.
!epsmch = dpmpar(1)
epsmch =epsilon(0._rlg)

info = 0
iflag = 0
nfev = 0
njev = 0

!  check the input parameters for errors.
if (n .gt. 0 .and. m .ge. n .and. ldfjac .ge. m       &
         .and. ftol .ge. zero .and. xtol .ge. zero .and. gtol .ge. zero   &
         .and. maxfev .gt. 0 .and. factor .gt. zero) then
  itest=1
  if (mode == 2) then
      do j = 1, n
         if (diag(j) .le. zero) itest=0
      enddo
  endif
  if(itest .ne.0)then

!     evaluate the function at the starting point
!     and calculate its norm.
    iflag = 1
    call fcn(m,n,x,fvec,fjac,ldfjac,iflag)
!    write(*,*)'dans lmder 1',iflag
!    write(*,*)'dans lmder 1','x',x
!    write(*,*)'dans lmder 1','fvec',fvec
!    write(*,*)'dans lmder 1','fjac',fjac
    nfev = 1
    if (iflag .ge. 0) then
      fnorm = enorm(m,fvec)

!       initialize levenberg-marquardt parameter and iteration counter.
      par = zero
      iter = 1

!       beginning of the outer loop.
      do 

!          calculate the jacobian matrix.
         iflag = 2
         call fcn(m,n,x,fvec,fjac,ldfjac,iflag)
!    write(*,*)'dans lmder 2',iflag
!    write(*,*)'dans lmder 2','x',x
!    write(*,*)'dans lmder 2','fvec',fvec
!    write(*,*)'dans lmder 2','fjac',fjac
         njev = njev + 1
         if (iflag .lt. 0) exit

!          if requested, call fcn to enable printing of iterates.
         if (nprint .gt. 0) then
            iflag = 0
            if (mod(iter-1,nprint)==0)call fcn(m,n,x,fvec,fjac,ldfjac,iflag)
!    write(*,*)'dans lmder 3',iflag
!    write(*,*)'dans lmder 3','x',x
!    write(*,*)'dans lmder 3','fvec',fvec
!    write(*,*)'dans lmder 3','fjac',fjac
            if (iflag .lt. 0) exit
         endif

!          compute the qr factorization of the jacobian.
         call qrfac(m,n,fjac,ldfjac,.true.,ipvt,n,wa1,wa2,wa3)
!    write(*,*)'aprs qrfac'
!    write(*,*)'dans lmder 4','fjac',fjac
!    write(*,*)'dans lmder 4','ipvt',ipvt
!    write(*,*)'dans lmder 4','wa1',wa1

!          on the first iteration and if mode is 1, scale according
!          to the norms of the columns of the initial jacobian.
         if (iter == 1) then
            if (mode .ne. 2) then
               do j = 1, n
                 diag(j) = wa2(j)
                 if (wa2(j) .eq. zero) diag(j) = one
               enddo
            endif

!             on the first iteration, calculate the norm of the scaled x
!             and initialize the step bound delta.
            wa3(1:n) = diag(1:n)*x(1:n)
            xnorm = enorm(n,wa3)
            delta = factor*xnorm
            if (delta .eq. zero) delta = factor
          endif

!           form (q transpose)*fvec and store the first n components in qtf.
          wa4(1:m) = fvec(1:m)
          do j = 1, n
            if (fjac(j,j) .ne. zero) then
               suml   = zero
               do i = j, m
                  suml   = suml   + fjac(i,j)*wa4(i)
               enddo
               temp = -suml  /fjac(j,j)
               wa4(j:m) = wa4(j:m) + fjac(j:m,j)*temp
             endif
             fjac(j,j) = wa1(j)
             qtf(j) = wa4(j)
          enddo

!           compute the norm of the scaled gradient.
          gnorm = zero
          if (fnorm .ne. zero) then
            do j = 1, n
               l = ipvt(j)
               if (wa2(l) .ne. zero) then
                 suml = zero
                 do i = 1, j
                    suml = suml + fjac(i,j)*(qtf(i)/fnorm)
                 enddo
                 gnorm = dmax1(gnorm,dabs(suml/wa2(l)))
               endif
            enddo
          endif

!           test for convergence of the gradient norm.
         if (gnorm .le. gtol) info = 4
         if (info .ne. 0) exit

!           rescale if necessary.
         if (mode .ne. 2) then
            do j = 1, n
                diag(j) = dmax1(diag(j),wa2(j))
            enddo
         endif

!     beginning of the inner loop.
  200    continue

!           determine the levenberg-marquardt parameter.
         call lmpar(n,fjac,ldfjac,ipvt,diag,qtf,delta,par,wa1,wa2,wa3,wa4)
!    write(*,*)'apres lmpar'
!    write(*,*)'dans lmder 5','fjac',fjac
!    write(*,*)'dans lmder 5','ipvt',ipvt
!    write(*,*)'dans lmder 5','par',par
!    write(*,*)'dans lmder 5','wa1',wa1
!    write(*,*)'dans lmder 5','wa2',wa2

!           store the direction p and x + p. calculate the norm of p.
         do j = 1, n
               wa1(j) = -wa1(j)
               wa2(j) = x(j) + wa1(j)
               wa3(j) = diag(j)*wa1(j)
         enddo
         pnorm = enorm(n,wa3)

!          on the first iteration, adjust the initial step bound.
         if (iter .eq. 1) delta = dmin1(delta,pnorm)

!          evaluate the function at x + p and calculate its norm.
         iflag = 1
         call fcn(m,n,wa2,wa4,fjac,ldfjac,iflag)
!    write(*,*)'dans lmder 6',iflag
!    write(*,*)'dans lmder 6','x',x
!    write(*,*)'dans lmder 6','fvec',fvec
!    write(*,*)'dans lmder 6','fjac',fjac
         nfev = nfev + 1
         if (iflag .lt. 0) exit
         fnorm1 = enorm(m,wa4)

!          compute the scaled actual reduction.
         actred = -one
         if (p1*fnorm1 .lt. fnorm) actred = one - (fnorm1/fnorm)**2

!          compute the scaled predicted reduction and
!          the scaled directional derivative.

         do j = 1, n
             wa3(j) = zero
             l = ipvt(j)
             temp = wa1(l)
             wa3(1:j) = wa3(1:j) + fjac(1:j,j)*temp
         enddo
         temp1 = enorm(n,wa3)/fnorm
         temp2 = (dsqrt(par)*pnorm)/fnorm
         prered = temp1**2 + temp2**2/p5
         dirder = -(temp1**2 + temp2**2)

!           compute the ratio of the actual to the predicted
!           reduction.
         ratio = zero
         if (prered .ne. zero) ratio = actred/prered

!           update the step bound.
         if (ratio .le. p25) then
            if (actred .ge. zero) temp = p5
            if (actred .lt. zero)temp = p5*dirder/(dirder + p5*actred)
            if (p1*fnorm1 .ge. fnorm .or. temp .lt. p1) temp = p1
            delta = temp*dmin1(delta,pnorm/p1)
            par = par/temp
         else
            if (par .eq. zero .or. ratio .ge. p75) then
               delta = pnorm/p5
               par = p5*par
            endif
         endif

!          test for successful iteration.
         if (ratio .ge. p0001) then

!             successful iteration. update x, fvec, and their norms.
            do j = 1, n
               x(j) = wa2(j)
               wa2(j) = diag(j)*x(j)
            enddo
            do  i = 1, m
               fvec(i) = wa4(i)
            enddo
            xnorm = enorm(n,wa2)
            fnorm = fnorm1
            iter = iter + 1
         endif

!           tests for convergence.
         if (dabs(actred) .le. ftol .and. prered .le. ftol   &
                   .and. p5*ratio .le. one) info = 1
         if (delta .le. xtol*xnorm) info = 2
         if (dabs(actred) .le. ftol .and. prered .le. ftol    &
               .and. p5*ratio .le. one .and. info .eq. 2) info = 3
         if (info .ne. 0) exit

!           tests for termination and stringent tolerances.
         if (nfev .ge. maxfev) info = 5
         if (dabs(actred) .le. epsmch .and. prered .le. epsmch    &
               .and. p5*ratio .le. one) info = 6
         if (delta .le. epsmch*xnorm) info = 7
         if (gnorm .le. epsmch) info = 8
         if (info .ne. 0) exit

!        end of the inner loop. repeat if iteration unsuccessful.
         if (ratio .lt. p0001) go to 200

!     end of the outer loop.
      enddo
    endif
  endif
endif

!  termination, either normal or user imposed.
if (iflag .lt. 0) info = iflag
iflag = 0
if (nprint .gt. 0) call fcn(m,n,x,fvec,fjac,ldfjac,iflag)

   END SUBROUTINE lmder

!!!!**************************************************************
!      function dpmpar(i)
!  **********

!  Function dpmpar
!  This function provides double precision machine parameters
!  when the appropriate set of data statements is activated (by
!  removing the c from column 1) and all other data statements are
!  rendered inactive. Most of the parameter values were obtained
!  from the corresponding Bell Laboratories Port Library function.

!  The function statement is
!    function dpmpar(i)
!  where
!    i is an integer input variable set to 1, 2, or 3 which
!      selects the desired machine parameter. If the machine has
!      t base b digits and its smallest and largest exponents are
!      emin and emax, respectively, then these parameters are
!      dpmpar(1) = b**(1 - t), the machine precision,
!      dpmpar(2) = b**(emin - 1), the smallest magnitude,
!      dpmpar(3) = b**emax*(1 - b**(-t)), the largest magnitude.

!  Argonne National Laboratory. MINPACK Project. November 1996.
!  Burton S. Garbow, Kenneth E. Hillstrom, Jorge J. More'
!  **********
  !! Arguments
!integer,INTENT(IN)                            :: i

   !! * Local declarations
!integer,dimension(4)                :: mcheps,minmag,maxmag
!real(kind=rlg),dimension(3)         :: dmach

!      equivalence (dmach(1),mcheps(1))
!      equivalence (dmach(2),minmag(1))
!      equivalence (dmach(3),maxmag(1))

!  Machine constants for IEEE machines.

!      data dmach(1) /2.22044604926d-16/
!      data dmach(2) /2.22507385852d-308/
!      data dmach(3) /1.79769313485d+308/

!      dpmpar = dmach(i)

!   END FUNCTION dpmpar


!****************************************************************************
#endif  
 END MODULE
