 MODULE reactions

   !!======================================================================
   !!                   ***  MODULE  REACTIONs           ***
   !!    contient les subroutines decrivant les echanges par reactions
   !!======================================================================
   !! * Modules used
USE parametres
USE com_morgan
USE com_varmilieu

IMPLICIT NONE
PRIVATE

   !! * Accessibility
PUBLIC partage     ! routine called by moco_morgan.F90
PUBLIC sink_source     ! routine called by moco_morgan.F90
PUBLIC volatil     ! routine called by moco_morgan.F90
PUBLIC sorption_cin     ! routine called by moco_morgan.F90

   !! * Shared module variables

   !! * Private variables

 CONTAINS

!!======================================================================
!
                SUBROUTINE partage
!&E---------------------------------------------------------------------
!         Partage dissous/particulaire entre les differentes especes       
!&E---------------------------------------------------------------------
   !! * Modules used
   
   !! Arguments

   !! * Local declarations
integer                :: i,iesp,isub,iter,ipartesp
real(kind=rlg)         :: somd,somp,somnl,som,csorb,ctot,difctot
real(kind=rlg),dimension(2)   :: ctotemp
real(kind=rlg)         :: aequ,bequ,cequ,delta,scdiss,scpart,binf,bsup
! ****************************************************************************
         
if(l_mode1c)then
  
!  sensibilite aux conditions du milieu
!  ::::::::::::::::::::::::::::::::::::::
  do i=1,ncalcul


!!!!calcul de la concentration de l espece dissoute libre
    somd=1._rlg
    somp=0._rlg
    somnl=0._rlg
    som=1._rlg
    do iesp=1,nesp_diss
      isub=isub_esp(iesp)
      csorb=xpart_esp(iesp)*csub_v(isub,i)
      som=som+csorb
      somd=somd+csorb
    enddo
    do iesp=nesp_diss+1,nesp_diss+nesp_part_eq
      isub=isub_esp(iesp)
      csorb=f_dom_l(iesp)*xpart_esp(iesp)*csub_v(isub,i)
      som=som+csorb
      somp=somp+csorb
      if(l_dom_nl)then
!!!!!!!!! MES associe
        ipartesp=ipart_sub(isub_esp(iesp))
!!!!!!!  il existe un ou des domaine non lineaires                 
!!!!!!!  modele DMM : un seul domaine non lineaire possible 
!!!!!!!  et une seule espece avec domaine non lineaire
        if(l_langmuir)then
            somnl=xpartnl(iesp,1)*f_dom_nl(iesp,1)*   &
                  xsorptnl(iesp,1)*cpart_v(ipartesp,i)
        endif
      endif
    enddo
         
!    si pas de domaine non lineaire
    if(.not.l_dom_nl)then
      cesp(nvar,i)=c_conta_ech(i)/som
      pcent_cesp(nvar,i)=cesp(nvar,i)/c_conta_ech(i)*100
      ctot=cesp(nvar,i)
!!!!  concentration des autres especes 
      do iesp=1,nespt_echeq
        isub=isub_esp(iesp)
        cesp(iesp,i)=cesp(nvar,i)*f_dom_l(iesp)*xpart_esp(iesp)*csub_v(isub,i)
        pcent_cesp(iesp,i)=cesp(iesp,i)/c_conta_ech(i)*100
        ctot=ctot+cesp(iesp,i)
      enddo
!!!!!!  coef de partage global 
!!!!!!  var(1,i)=MEStotales
      xpartg(i)=somp/var_milieu(1,i)/somd

    else
!     domaine lineaire
      if(l_langmuir)then
!!!!!!  modele DMM : resolution d'une equation du second degr
!!!!!!  une seule espece (la premire) avec un seul domaine non linaire
        iesp=nesp_diss+1
        aequ=xpartnl(iesp,1)*(somd+somp)
        bequ=somd+somp+somnl-c_conta_ech(i)*xpartnl(iesp,1)
        cequ=-c_conta_ech(i)
        delta=bequ**2-4*aequ*cequ
        cesp(nvar,i)=(-bequ+sqrt(delta))*0.5/aequ
        if(cesp(nvar,i)<0)cesp(nvar,i)=(-bequ-sqrt(delta))*0.5/aequ
        pcent_cesp(nvar,i)=cesp(nvar,i)/c_conta_ech(i)*100
        ctot=cesp(nvar,i)
        scdiss=cesp(nvar,i)
        scpart=0._rlg

!  concentration des autres especes a l'equilibre
        do iesp=1,nesp_diss
          isub=isub_esp(iesp)
          cesp(iesp,i)=cesp(nvar,i)*xpart_esp(iesp)*csub_v(isub,i)
          pcent_cesp(iesp,i)=cesp(iesp,i)/c_conta_ech(i)*100
          scdiss=scdiss+cesp(iesp,i)
        enddo
        do iesp=nesp_diss+1,nesp_part_eq+nesp_diss
          isub=isub_esp(iesp)
          ipartesp=ipart_sub(isub)
          cesp(iesp,i)=cesp(nvar,i)*                             &
              (f_dom_l(iesp)*xpart_esp(iesp)*csub_v(isub,i)+    &
               f_dom_nl(iesp,1)*xpartnl(iesp,1)*xsorptnl(iesp,1)    &
              *cpart_v(ipartesp,i)/(1+xpartnl(iesp,1)*cesp(nvar,i)))
          pcent_cesp(iesp,i)=cesp(iesp,i)/c_conta_ech(i)*100
          scpart=scpart+cesp(iesp,i)
        enddo
        ctot=scdiss+scpart
        difctot=ctot-c_conta_ech(i)
        cesp(nvar,i)=cesp(nvar,i)-difctot
        pcent_cesp(nvar,i)=cesp(nvar,i)/c_conta_ech(i)*100
        scdiss=scdiss-difctot
        if(abs(difctot).gt.c_conta_ech(i)/100.)then
            write(*,*)'attention'
            write(*,*)'la concentration totale n est pas conserve'
            write(*,*) ctot,c_conta_ech(i)
            write(*,*) ' t= ',t,' ic= ',i
        endif
!!!!!!!  coef de partage global 
!!!!!!!  var(1,i)=MEStotales
        xpartg(i)=scdiss/var_milieu(1,i)/scpart
           
      else 
!!!!!!  modele DRM :rsolution par itration
        aequ=somd+somp
        binf=0.
        bsup=c_conta_ech(i)
        cesp(nvar,i)=(binf+bsup)*0.5
        ctot=aequ*cesp(nvar,i)
        do iesp=nesp_diss+1,nesp_part_eq+nesp_diss
          isub=isub_esp(iesp)
          ipartesp=ipart_sub(isub)
          ctotemp(1:n_dom_nl(iesp))=  &
               f_dom_nl(iesp,1:n_dom_nl(iesp))*xpartnl(iesp,1:n_dom_nl(iesp))* &
               cpart_v(ipartesp,i)*cesp(nvar,i)**xsorptnl(iesp,1:n_dom_nl(iesp))
          ctot=sum(ctotemp)
        enddo      
        difctot=ctot-c_conta_ech(i)
        iter=0
        do while (abs(difctot).gt.epsnl)
          if(difctot.gt.0.)then
               bsup=cesp(nvar,i)
          else
               binf=cesp(nvar,i)
          endif
          cesp(nvar,i)=+(bsup+binf)*0.5
          ctot=aequ*cesp(nvar,i)
          do iesp=nesp_diss+1,nesp_part_eq+nesp_diss
            isub=isub_esp(iesp)
            ipartesp=ipart_sub(isub)
            ctotemp(1:n_dom_nl(iesp))=  &
               +f_dom_nl(iesp,1:n_dom_nl(iesp))*xpartnl(iesp,1:n_dom_nl(iesp))* &
               cpart_v(ipartesp,i)*cesp(nvar,i)**xsorptnl(iesp,1:n_dom_nl(iesp))
            ctot=sum(ctotemp)
          enddo      
          difctot=ctot-c_conta_ech(i) 
          iter=iter+1
          if(iter.eq.200)then
            write(*,*)'attention iter>200'
          else if (iter.gt.2000)then
            write(*,*)'iter>2000'
            stop
          endif 
        enddo
        pcent_cesp(nvar,i)=cesp(nvar,i)/c_conta_ech(i)*100
        scdiss=cesp(nvar,i)
        scpart=0.
!!!!!!!! concentration des autres especes a l equilibre
        do iesp=1,nesp_diss
          isub=isub_esp(iesp)
          cesp(iesp,i)=cesp(nvar,i)*xpart_esp(iesp)*csub_v(isub,i)
          pcent_cesp(iesp,i)=cesp(iesp,i)/c_conta_ech(i)*100
          scdiss=scdiss+cesp(iesp,i)
        enddo
        do iesp=nesp_diss+1,nesp_diss+nesp_part_eq
          isub=isub_esp(iesp)
          ipartesp=ipart_sub(isub)
          ctotemp(1:n_dom_nl(iesp))=  &
            +f_dom_nl(iesp,1:n_dom_nl(iesp))*xpartnl(iesp,1:n_dom_nl(iesp))*  &
            cpart_v(ipartesp,i)*cesp(nvar,i)**xsorptnl(iesp,1:n_dom_nl(iesp))
          somnl=sum(ctotemp)
          cesp(iesp,i)=somnl+ cesp(nvar,i)*f_dom_l(iesp)*xpart_esp(iesp)*csub_v(isub,i)
          pcent_cesp(iesp,i)=cesp(iesp,i)/c_conta_ech(i)*100
          scpart=scpart+cesp(iesp,i)
        enddo
!!!!     correction de l erreur
        ctot=scdiss+scpart
        difctot=ctot-c_conta_ech(i)
        cesp(nvar,i)=cesp(nvar,i)-difctot
        pcent_cesp(nvar,i)=cesp(nvar,i)/c_conta_ech(i)*100
        scdiss=scdiss-difctot
!!!!!!!  coef de partage global 
!!!!!!!  var(1,i)=MEStotales
        xpartg(i)=scpart/var_milieu(1,i)/scdiss
      endif
 
    endif
  enddo

! sensibilite au contaminant
!  ::::::::::::::::::::::::::::::::::::::
else 
      
  do i=1,ncalcul
!!  calcul de la concentration de lespece dissoute libre
    somd=1
    somp=0
    somnl=0
    som=1
    do iesp=1,nesp_diss
      isub=isub_esp(iesp)
      csorb=xpart_espv(iesp,i)*csub(isub)
      som=som+csorb
      somd=somd+csorb
    enddo
    do iesp=nesp_diss+1,nesp_diss+nesp_part_eq
      isub=isub_esp(iesp)
      csorb=f_dom_l(iesp)*xpart_espv(iesp,i)*csub(isub)
      som=som+csorb
      somp=somp+csorb
      if(l_dom_nl)then
!!!!!!!!!! MES associe
        ipartesp=ipart_sub(isub_esp(iesp))
!!        il existe un ou des domaine non lineaires                 
!!        modele DMM : un seul domaine non lineaire possible
        if(l_langmuir)then
          somnl=xpartnl(iesp,1)*f_dom_nl(iesp,1)*   &
                xsorptnl(iesp,1)*cpart(ipartesp)
        endif
      endif
    enddo
         
!! pas de domaine non lineaire         
    if(.not.l_dom_nl)then
      cesp(nvar,i)=c_conta_ech(i)/som
      pcent_cesp(nvar,i)=cesp(nvar,i)/c_conta_ech(i)*100
      ctot=cesp(nvar,i)
!!!!!  concentration des autres especes 
      do iesp=1,nespt_echeq
        isub=isub_esp(iesp)
        cesp(iesp,i)=cesp(nvar,i)*xpart_espv(iesp,i)*csub(isub)
        pcent_cesp(iesp,i)=cesp(iesp,i)/c_conta_ech(i)*100
        ctot=ctot+cesp(iesp,i)
      enddo
!!!!!!  coef de partage global 
!!!!!!  var(1,i)=MEStotales
      xpartg(i)=somp/c_MESt/somd

    else 
!!!!!!! modele DMM : resolution d'une equation du second degr
      if(l_langmuir)then 
        iesp=nesp_diss+1               
        aequ=xpartnl(iesp,1)*(somd+somp)
        bequ=somd+somp+somnl-c_conta_ech(i)*xpartnl(iesp,1)
        cequ=-c_conta_ech(i)
        delta=bequ**2-4*aequ*cequ
        cesp(nvar,i)=(-bequ+sqrt(delta))*0.5/aequ
        if(cesp(nvar,i).lt.0)cesp(nvar,i)=(-bequ-sqrt(delta))*0.5/aequ
        pcent_cesp(nvar,i)=cesp(nvar,i)/c_conta_ech(i)*100
        scdiss=cesp(nvar,i)
        scpart=0.
!!!!!!!!  concentration des autres especes a l'equilibre
        do iesp=1,nesp_diss
          isub=isub_esp(iesp)
          cesp(iesp,i)=cesp(nvar,i)*xpart_espv(iesp,i)*csub(isub)
          pcent_cesp(iesp,i)=cesp(iesp,i)/c_conta_ech(i)*100
          scdiss=scdiss+cesp(iesp,i)
        enddo
        do iesp=nesp_diss+1,nesp_part_eq+nesp_diss
          isub=isub_esp(iesp)
          ipartesp=ipart_sub(isub)
          cesp(iesp,i)=cesp(nvar,i)*(f_dom_l(iesp)*xpart_espv(iesp,i)*csub(isub)+ &
                f_dom_nl(iesp,1)*xpartnl(iesp,1)*    &
                xsorptnl(iesp,1)*cpart(ipartesp) /(1+xpartnl(iesp,1)*cesp(nvar,i)))

           pcent_cesp(iesp,i)=cesp(iesp,i)/c_conta_ech(i)*100
           scpart=scpart+cesp(iesp,i)
         enddo
         ctot=scdiss+scpart
         difctot=ctot-c_conta_ech(i)
         cesp(nvar,i)=cesp(nvar,i)-difctot
         if(abs(difctot).gt.c_conta_ech(i)/100.)then
             write(*,*)'attention'
             write(*,*)'la concentration totale n est pas conserve'
             write(*,*) ctot,c_conta_ech(i)
             write(*,*) ' t= ',t,' ic= ',i
         endif
         pcent_cesp(nvar,i)=cesp(nvar,i)/c_conta_ech(i)*100
         scdiss=scdiss-difctot
!!!        coef de partage global 
!!!       var(1,i)=MEStotales
         xpartg(i)=scpart/c_MESt/scdiss
            
      else 
         
!!!!!!!! modele DRM :rsolution par itration
        aequ=somd+somp
        binf=0.
        bsup=c_conta_ech(i)
        cesp(nvar,i)=(binf+bsup)*0.5
        ctot=aequ*cesp(nvar,i)
        do iesp=nesp_diss+1,nesp_part_eq+nesp_diss
          isub=isub_esp(iesp)
          ipartesp=ipart_sub(isub)
          ctotemp(1:n_dom_nl(iesp))=   &
            f_dom_nl(iesp,1:n_dom_nl(iesp))*xpartnl(iesp,1:n_dom_nl(iesp))*  &
            cpart(ipartesp)*cesp(nvar,i)**xsorptnl(iesp,1:n_dom_nl(iesp))
          ctot=sum(ctotemp)
        enddo      
        difctot=ctot-c_conta_ech(i)
        iter=0
        do while (abs(difctot).gt.epsnl)
          if(difctot.gt.0.)then
               bsup=cesp(nvar,i)
          else
               binf=cesp(nvar,i)
          endif
          cesp(nvar,i)=+(bsup+binf)*0.5
          ctot=aequ*cesp(nvar,i)
          do iesp=nesp_diss+1,nesp_part_eq+nesp_diss
            isub=isub_esp(iesp)
            ipartesp=ipart_sub(isub)
             ctotemp(1:n_dom_nl(iesp))=  &
               f_dom_nl(iesp,1:n_dom_nl(iesp))*xpartnl(iesp,1:n_dom_nl(iesp))*  &
                 cpart(ipartesp)*cesp(nvar,i)**xsorptnl(iesp,1:n_dom_nl(iesp))
             ctot=sum(ctotemp)
          enddo      
          difctot=ctot-c_conta_ech(i) 
          iter=iter+1
          if(iter.eq.200)then
            write(*,*)'attention iter>200'
          else if (iter.gt.2000)then
            write(*,*)'iter>2000'
            stop
          endif 
        enddo
        pcent_cesp(nvar,i)=cesp(nvar,i)/c_conta_ech(i)*100
        scdiss=cesp(nvar,i)
        scpart=0.
!!!!!!!  concentration des autres especes a l equilibre
        do iesp=1,nesp_diss
          isub=isub_esp(iesp)
          cesp(iesp,i)=cesp(nvar,i)*xpart_espv(iesp,i)*csub(isub)
          pcent_cesp(iesp,i)=cesp(iesp,i)/c_conta_ech(i)*100
          scdiss=scdiss+cesp(iesp,i)
        enddo
        do iesp=nesp_diss+1,nesp_diss+nesp_part_eq
          isub=isub_esp(iesp)
          ipartesp=ipart_sub(isub)
          ctotemp(1:n_dom_nl(iesp))=  &
             f_dom_nl(iesp,1:n_dom_nl(iesp))*xpartnl(iesp,1:n_dom_nl(iesp))*    &
             cpart(ipartesp)*cesp(nvar,i)**xsorptnl(iesp,1:n_dom_nl(iesp))
          somnl=sum(ctotemp)
          cesp(iesp,i)=somnl+ cesp(nvar,i)*f_dom_l(iesp)*xpart_espv(iesp,i)*csub(isub)
          pcent_cesp(iesp,i)=cesp(iesp,i)/c_conta_ech(i)*100
          scpart=scpart+cesp(iesp,i)
        enddo
!!!!!!!!!! correction de l erreur
        ctot=scdiss+scpart
        difctot=ctot-c_conta_ech(i)
        cesp(nvar,i)=cesp(nvar,i)-difctot
        pcent_cesp(nvar,i)=cesp(nvar,i)/c_conta_ech(i)*100
        scdiss=scdiss-difctot
!!       coef de partage global 
!!       var(1,i)=MEStotales
        xpartg(i)=scpart/c_MESt/scdiss
      endif

    endif      
  enddo
endif      
  END SUBROUTINE partage

!!======================================================================
!
                SUBROUTINE sink_source(ic)
!&E---------------------------------------------------------------------
!          Reactions generiques  
!&E---------------------------------------------------------------------
   !! * Modules used
   
   !! * Local declarations
 integer                          :: ire,ivr,iv,ic,iflim,ivfl
 real(kind=rlg)                   :: cin_reac,xdtemp,xflim1,xflim2,xcv

!**************************************************************************** 
do ire=1,nb_reac

  sksc(ire,1:nvar)=0.
  if(l_mode1c)then
    cin_reac=cin_max_re(ire)
  else
    cin_reac=cin_max_rev(ire,ic)
  endif
   
!     correction de la cinetique avec la temperature
  if(l_arrh(ire)) then
    xdtemp=(temperaC-tempref_arrh(ire))/xeta_sigma_arrh(ire)
    cin_reac=cin_reac*(1-ityp_arrh(ire))*teta_arrh(ire)**xdtemp   &
            +ityp_arrh(ire)*teta_arrh(ire)*exp(-xdtemp**2)
  endif

!     fonctions limitantes
  xflim1=1.0
  do iflim=1,nb_flim1(ire)
    ivfl=irkm_flim1(ire,iflim)
    if(ivfl.eq.-1)then
      xflim1=xflim1*abs(salinite/(salinite+cte_flim1(ire,iflim)))
    else if(ivfl.eq.-2)then
      xflim1=xflim1*abs(var_milieu(1,ic)/(var_milieu(1,ic)+cte_flim1(ire,iflim)))
    else
      xflim1=xflim1*abs(cesp(ivfl,ic)/(cesp(ivfl,ic)+cte_flim1(ire,iflim)))
    endif
  enddo
  xflim2=1.0
  do iflim=1,nb_flim2(ire)
    ivfl=irkm_flim2(ire,iflim)
    if(ivfl.eq.-1)then
      xflim2=xflim2*abs(cte_flim2(ire,iflim)/(salinite+cte_flim2(ire,iflim)))
    else if(ivfl.eq.-2)then
      xflim2=xflim2*abs(cte_flim2(ire,iflim)/(var_milieu(1,ic)+cte_flim2(ire,iflim)))
    else
      xflim2=xflim2*abs(cte_flim2(ire,iflim)/(cesp(ivfl,ic)+cte_flim2(ire,iflim)))
    endif
  enddo

!      concentrations actives dans le flux d echange 
!          (si exposant different de 0)
  xcv=1.0_rlg
  do iv=1,nb_var_re(ire)
    xcv=xcv*cesp(irkm_var_re(ire,iv),ic)**expo_var(ire,iv)
  enddo
   
!      calcul du terme soupui quand le terme xcv=C^n.C^m est calcule
  do ivr=1,nb_var_re(ire)
     iv=irkm_var_re(ire,ivr)
     sksc(ire,iv)=sksc(ire,iv)+coef_stochio(ire,ivr)*cin_reac*xflim1*xflim2*xcv
  enddo

enddo

  END SUBROUTINE sink_source

!!======================================================================
!
                SUBROUTINE volatil(ic)
!&E---------------------------------------------------------------------
!           Volatilisation de l'espece volatilisable iesp_volat
!&E---------------------------------------------------------------------
   !! * Modules used
   
   !! * Local declarations
 integer                          :: ire,ivr,iv,ic,iflim,ivfl
 real(kind=rlg)                   :: temperaCi,temperaKi,sali,uci,ventci,zci
 real(kind=rlg)                   :: xkgh2o,xkg,xkldo2,xkldco2,xklvco2,xkl,sc_co2,xx

      
if(l_mode1c)then
 temperaCi=var_milieu(2,ic)
 temperaKi=var_milieu(2,ic)+273.15
 sali=var_milieu(3,ic)
 uci=var_milieu(4,ic)
 ventci=var_milieu(5,ic)
 zci=var_milieu(6,ic)
 if(henry_adim.eq.0)then
   henry_adimi=henry/cst_gas_univ/temperaKi
 else
   henry_adimi=henry_adim
 endif
       
else 
  temperaCi=temperaC
  temperaKi=temperaC+273.15
  sali=salinite
  uci=uc
  ventci=ventc
  zci=zc
  diffusivite=diffusivitev(ic)
  pdmolec=pdmolecv(ic)
  xnb_schmidt=xnb_schmidtv(ic)
  if(henry_adimv(ic).eq.0)then
    henry_adimi=henryv(ic)/cst_gas_univ/temperaKi
  else
    henry_adimi=henry_adimv(ic)
  endif
  coefH_staudinger=coefH_staudingerv(ic)
  cte_salt=cte_saltv(ic)       
endif
      
! phase gazeuse : compose de reference = H2O
!=============================================
!!!! formule de Schwartzenbach page 897
xkgh2o=(0.2_rlg*ventci+0.3)/100._rlg
!  si Yg=masse molaire      ==>> ich_yg=0
!  si Yl=diffusivit        ==>> ich_yg=1
if(.not.l_volkg_cor .or. diffusivite==0.)then              
  xkg=xkgh2o*(pdmolec/massmol_h2o)**cf_expvol_gas
else
  xkg=xkgh2o*(diffusivite/cf_difair_h2o)**cf_expvol_gas
endif            
            
! phase liquide : compose de reference = CO2
!=============================================
!  terme de diffusion turbulente (kld) et engendre par le vent (klv)
!  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if(l_volkl_dif)then
!!! formule de Churchill 
  xkldo2=5.8e-5_rlg*uci**0.969_rlg/zci**0.673_rlg
else 
!!! formule de O'Connor 
  xkldo2=4.24e-5_rlg*uci**0.5_rlg/zci**0.5_rlg
endif
!!! formule de Wen et al, 1984 si effet du sel
if(l_volkl_sal)then
   xkldo2=xkldo2*exp(0.0127_rlg*sali)
endif
!!! effet de la temprature - formule de Elmore et West, in Wilcock 1982
xkldo2=xkldo2*1.024_rlg**(temperaCi-20._rlg)
xkldco2=xkldo2*(cf_difwat_co2/cf_difwat_o2)**0.5_rlg 
if(ich_volkl_wind == 1)then
!!! formule de  Schwartzenbach page 905
  xklvco2=4.e-6_rlg+4.e-7_rlg*ventci**2 
else IF(ich_volkl_wind == 2)then
!!! formule de Wanninkhof 
  xklvco2=1.25e-6_rlg*ventci**1.64_rlg 
ELSE IF(ich_volkl_wind == 3)then
! formule de Nightingale 
  xklvco2=7.0619e-07_rlg*ventci+2.777e-06_rlg*ventci**2 
endif
           
!!!  si Yl=masse molaire      ==>> ich_yl=0
!!!  si Yl=nombre de schmidt  ==>> ich_yl=1
if(.not.l_volkl_cor.or.xnb_schmidt.eq.0)then
!   xkl=(xkldco2+xklvco2)  
!   xx= pdmolec/massmol_co2 
!   xx=xx**cf_expvol_liq
!   xkl=xkl*xx      
   xkl=(xkldco2+xklvco2)*(pdmolec/massmol_co2)**cf_expvol_liq
else
!!! calcul du nombre de Schmidt pour CO2 en fonction de la temperature
   sc_co2=exp(-0.052_rlg*temperaKi+21.71_rlg)
   xkl=(xkldco2+xklvco2)*(xnb_schmidt/sc_co2)**cf_expvol_liq
endif
       
!!! calcul de la constante de Henry en fonction de la temperature 
!!!   formule de Staudinger et Roberts (2001)
if(coefH_staudinger.ne.0.and. TRIM(ADJUSTL(ADJUSTR(name_conta)))/='Hg')then      
     henry_adimi=henry_adimi*10**(-coefH_staudinger*(1._rlg/temperaKi-1./293._rlg))
else if(name_conta=='Hg')then
     if(coefH_staudinger==0)then
!! formule de Clever et al(1985) utilise par Wangberg et al 2001
        henry_adimi=exp(-4633.3/temperaKi+14.53)
     else   
!! formule de Poissant d'aprs Sanemasa(1975)
        henry_adimi=0.0074*temperaCi+0.1551
      endif
endif 
!!! correction de la constante de Henry en fonction de la salinite 
henry_adimi=henry_adimi*10**(cte_salt*sali/68.35_rlg)
       
! concentration dans l'air suppose ne pas voluer 
! et toujours gale  la concentration initiale
xkol=xkl*xkg*henry_adimi/(xkg*henry_adimi+xkl)

                           
  END SUBROUTINE volatil

!!======================================================================
!
                SUBROUTINE sorption_cin
!&E---------------------------------------------------------------------
!           calcul de la desorption/sorption traite en CINETIQUE
!&E---------------------------------------------------------------------
   !! * Modules used
   
   !! * Local declarations
 integer                          :: ire,ivr,iv,ic,iflim,ivfl,i,ici,iesp
 integer                          :: iassoc,ipartesp
 real(kind=rlg)                   :: temperaCi,temperaKi,sali,uci,ventci,zci,cesp_eq,ff
 real(kind=rlg)                   :: xkgh2o,xkg,xkldo2,xkldco2,xklvco2,xkl,sc_co2,echg_ne
       
      
if(l_mode1c)then

!  sensibilite aux conditions du milieu
!  ::::::::::::::::::::::::::::::::::::::

do i=1,ncalcul
  do ici=1,nesp_part_ci
    iesp=nespt_echeq+ici
    iassoc=iesp_assoc(iesp)
    ipartesp=ipart_sub(isub_esp(iassoc))
    if(ichoix_ceqci(ici)==0)then
     ! ceq_ci a ete donne M/kg
        ff=cpart_v(ipartesp,i)
    else if(ichoix_ceqci(ici)==1)then
     ! ceq_ci a ete donne en fraction de la concentration initiale totale
        ff=c_conta_tot_init(i)
    else if(ichoix_ceqci(ici)==2)then
     ! ceq_ci /2 pour avoir un transfert progressif et 2 conc gales=conc equi 
        ff=1.
        ceq_ci(ici)=cesp(iassoc,i)
    endif
    cesp_eq=ceq_ci(ici)*ff
    echg_ne=cte_vit(ici)*(cesp(iesp,i)-cesp_eq)*dt
    cesp(iesp,i)=cesp(iesp,i)-echg_ne
    cesp(iassoc,i)=cesp(iassoc,i)+echg_ne
    if(cesp(iesp,i)<0. .or. cesp(iassoc,i)<0.)then
      write(*,*)'ATTENTION : le pas de temps est trop grand'
      write(*,*)'par rapport  la constante de vitesse de la '
      write(*,*)'raction de sorption en cinetique'
        write(*,*)'DIMINUER LE PAS DE TEMPS dans le fichier'
        write(*,*)name_file_varmilsens
      STOP
    endif
  enddo         
 enddo
 
! sensibilite au contaminant
!  ::::::::::::::::::::::::::::::::::::::
else 
      
 do i=1,ncalcul
   do ici=1,nesp_part_ci
     iesp=nespt_echeq+ici  
     iassoc=iesp_assoc(iesp)
     ipartesp=ipart_sub(isub_esp(iassoc))
     if(ichoix_ceqci(ici)==2)then
        cesp_eq=cesp(iassoc,i)
     else if(ichoix_ceqci(ici)==0)then
        cesp_eq=ceq_civ(ici,i)*cpart(ipartesp)
     else if(ichoix_ceqci(ici)==1)then
        cesp_eq=ceq_civ(ici,i)*c_conta_tot_init(i)
     endif
      echg_ne=cte_vitv(ici,i)*(cesp(iesp,i)-cesp_eq)*dt
      cesp(iesp,i)=cesp(iesp,i)-echg_ne
      cesp(iassoc,i)=cesp(iassoc,i)+echg_ne            
      if(cesp(iesp,i)<0. .or. cesp(iassoc,i)<0.)then
        write(*,*)'ATTENTION : le pas de temps est trop grand'
        write(*,*)'par rapport  la constante de vitesse de la '
        write(*,*)'raction de sorption en cinetique'
        write(*,*)'DIMINUER LE PAS DE TEMPS dans le fichier'
        write(*,*)name_file_varmilfix
        STOP
      endif
   enddo
 enddo

endif
  END SUBROUTINE sorption_cin
!!======================================================================
!
!****************************************************************************
 END MODULE
  